#
# Copyright 2012 New Dream Network, LLC (DreamHost)
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Test listing resources.
"""

import datetime
import json

import six
import webtest.app

from ceilometer.publisher import utils
from ceilometer import sample
from ceilometer.tests.functional.api import v2


class TestListResources(v2.FunctionalTest):

    def test_empty(self):
        data = self.get_json('/resources')
        self.assertEqual([], data)

    def _verify_resource_timestamps(self, res, first, last):
        # Bounds need not be tight (see ceilometer bug #1288372)
        self.assertIn('first_sample_timestamp', res)
        self.assertTrue(first.isoformat() >= res['first_sample_timestamp'])
        self.assertIn('last_sample_timestamp', res)
        self.assertTrue(last.isoformat() <= res['last_sample_timestamp'])

    def test_instance_no_metadata(self):
        timestamp = datetime.datetime(2012, 7, 2, 10, 40)
        sample1 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id',
            'project-id',
            'resource-id',
            timestamp=timestamp,
            resource_metadata=None,
            source='test',
        )
        msg = utils.meter_message_from_counter(
            sample1, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg)

        data = self.get_json('/resources')
        self.assertEqual(1, len(data))
        self._verify_resource_timestamps(data[0], timestamp, timestamp)

    def test_instances(self):
        timestamps = {
            'resource-id': datetime.datetime(2012, 7, 2, 10, 40),
            'resource-id-alternate': datetime.datetime(2012, 7, 2, 10, 41),
        }
        sample1 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id',
            'project-id',
            'resource-id',
            timestamp=timestamps['resource-id'],
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample',
                               },
            source='test',
        )
        msg = utils.meter_message_from_counter(
            sample1, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg)

        sample2 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id',
            'project-id',
            'resource-id-alternate',
            timestamp=timestamps['resource-id-alternate'],
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample2',
                               },
            source='test',
        )
        msg2 = utils.meter_message_from_counter(
            sample2, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg2)

        data = self.get_json('/resources')
        self.assertEqual(2, len(data))
        for res in data:
            timestamp = timestamps.get(res['resource_id'])
            self._verify_resource_timestamps(res, timestamp, timestamp)

    def test_instance_multiple_samples(self):
        timestamps = [
            datetime.datetime(2012, 7, 2, 10, 41),
            datetime.datetime(2012, 7, 2, 10, 42),
            datetime.datetime(2012, 7, 2, 10, 40),
        ]
        for timestamp in timestamps:
            datapoint = sample.Sample(
                'instance',
                'cumulative',
                '',
                1,
                'user-id',
                'project-id',
                'resource-id',
                timestamp=timestamp,
                resource_metadata={'display_name': 'test-server',
                                   'tag': 'self.sample-%s' % timestamp,
                                   },
                source='test',
            )
            msg = utils.meter_message_from_counter(
                datapoint,
                self.CONF.publisher.telemetry_secret,
            )
            self.conn.record_metering_data(msg)

        data = self.get_json('/resources')
        self.assertEqual(1, len(data))
        self._verify_resource_timestamps(data[0],
                                         timestamps[-1], timestamps[1])

    def test_instances_one(self):
        sample1 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id',
            'project-id',
            'resource-id',
            timestamp=datetime.datetime(2012, 7, 2, 10, 40),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample',
                               },
            source='test',
        )
        msg = utils.meter_message_from_counter(
            sample1, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg)

        sample2 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id',
            'project-id',
            'resource-id-alternate',
            timestamp=datetime.datetime(2012, 7, 2, 10, 41),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample2',
                               },
            source='test',
        )
        msg2 = utils.meter_message_from_counter(
            sample2, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg2)

        data = self.get_json('/resources/resource-id')
        self.assertEqual('resource-id', data['resource_id'])

    def test_with_source(self):
        sample1 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id',
            'project-id',
            'resource-id',
            timestamp=datetime.datetime(2012, 7, 2, 10, 40),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample',
                               },
            source='test_list_resources',
        )
        msg = utils.meter_message_from_counter(
            sample1, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg)

        sample2 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id2',
            'project-id',
            'resource-id-alternate',
            timestamp=datetime.datetime(2012, 7, 2, 10, 41),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample2',
                               },
            source='not-test',
        )
        msg2 = utils.meter_message_from_counter(
            sample2, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg2)

        data = self.get_json('/resources', q=[{'field': 'source',
                                               'value': 'test_list_resources',
                                               }])
        ids = [r['resource_id'] for r in data]
        self.assertEqual(['resource-id'], ids)
        sources = [r['source'] for r in data]
        self.assertEqual(['test_list_resources'], sources)

    def test_with_invalid_resource_id(self):
        sample1 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id',
            'project-id',
            'resource-id-1',
            timestamp=datetime.datetime(2012, 7, 2, 10, 40),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample',
                               },
            source='test_list_resources',
        )
        msg = utils.meter_message_from_counter(
            sample1, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg)

        sample2 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id2',
            'project-id',
            'resource-id-2',
            timestamp=datetime.datetime(2012, 7, 2, 10, 41),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample2',
                               },
            source='test_list_resources',
        )
        msg2 = utils.meter_message_from_counter(
            sample2, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg2)

        resp1 = self.get_json('/resources/resource-id-1')
        self.assertEqual("resource-id-1", resp1["resource_id"])

        resp2 = self.get_json('/resources/resource-id-2')
        self.assertEqual("resource-id-2", resp2["resource_id"])

        resp3 = self.get_json('/resources/resource-id-3', expect_errors=True)
        self.assertEqual(404, resp3.status_code)
        json_data = resp3.body
        if six.PY3:
            json_data = json_data.decode('utf-8')
        self.assertEqual("Resource resource-id-3 Not Found",
                         json.loads(json_data)['error_message']
                         ['faultstring'])

    def test_with_user(self):
        sample1 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id',
            'project-id',
            'resource-id',
            timestamp=datetime.datetime(2012, 7, 2, 10, 40),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample',
                               },
            source='test_list_resources',
        )
        msg = utils.meter_message_from_counter(
            sample1, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg)

        sample2 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id2',
            'project-id',
            'resource-id-alternate',
            timestamp=datetime.datetime(2012, 7, 2, 10, 41),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample2',
                               },
            source='not-test',
        )
        msg2 = utils.meter_message_from_counter(
            sample2, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg2)

        data = self.get_json('/resources', q=[{'field': 'user_id',
                                               'value': 'user-id',
                                               }])
        ids = [r['resource_id'] for r in data]
        self.assertEqual(['resource-id'], ids)

    def test_with_project(self):
        sample1 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id',
            'project-id',
            'resource-id',
            timestamp=datetime.datetime(2012, 7, 2, 10, 40),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample',
                               },
            source='test_list_resources',
        )
        msg = utils.meter_message_from_counter(
            sample1, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg)

        sample2 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id2',
            'project-id2',
            'resource-id-alternate',
            timestamp=datetime.datetime(2012, 7, 2, 10, 41),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample2',
                               },
            source='not-test',
        )
        msg2 = utils.meter_message_from_counter(
            sample2, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg2)

        data = self.get_json('/resources', q=[{'field': 'project_id',
                                               'value': 'project-id',
                                               }])
        ids = [r['resource_id'] for r in data]
        self.assertEqual(['resource-id'], ids)

    def test_with_user_non_admin(self):
        sample1 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id2',
            'project-id2',
            'resource-id-alternate',
            timestamp=datetime.datetime(2012, 7, 2, 10, 41),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample1',
                               },
            source='not-test',
        )
        msg2 = utils.meter_message_from_counter(
            sample1, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg2)

        data = self.get_json('/resources',
                             headers={"X-Roles": "Member",
                                      "X-Project-Id": "project-id2"})
        ids = set(r['resource_id'] for r in data)
        self.assertEqual(set(['resource-id-alternate']), ids)

    def test_with_user_wrong_tenant(self):
        sample1 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id2',
            'project-id2',
            'resource-id-alternate',
            timestamp=datetime.datetime(2012, 7, 2, 10, 41),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample1',
                               },
            source='not-test',
        )
        msg2 = utils.meter_message_from_counter(
            sample1, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg2)

        data = self.get_json('/resources',
                             headers={"X-Roles": "Member",
                                      "X-Project-Id": "project-wrong"})
        ids = set(r['resource_id'] for r in data)
        self.assertEqual(set(), ids)

    def test_metadata(self):
        sample1 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id',
            'project-id',
            'resource-id',
            timestamp=datetime.datetime(2012, 7, 2, 10, 40),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample',
                               'dict_properties': {'key.$1': {'$key': 'val'}},
                               'not_ignored_list': ['returned'],
                               },
            source='test',
        )
        msg = utils.meter_message_from_counter(
            sample1, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg)

        data = self.get_json('/resources')
        metadata = data[0]['metadata']
        self.assertEqual([(u'dict_properties.key:$1:$key', u'val'),
                          (u'display_name', u'test-server'),
                          (u'not_ignored_list', u"['returned']"),
                          (u'tag', u'self.sample')],
                         list(sorted(six.iteritems(metadata))))

    def test_resource_meter_links(self):
        sample1 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id',
            'project-id',
            'resource-id',
            timestamp=datetime.datetime(2012, 7, 2, 10, 40),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample',
                               },
            source='test_list_resources',
        )
        msg = utils.meter_message_from_counter(
            sample1, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg)

        data = self.get_json('/resources')
        links = data[0]['links']
        self.assertEqual(2, len(links))
        self.assertEqual('self', links[0]['rel'])
        self.assertTrue((self.PATH_PREFIX + '/resources/resource-id')
                        in links[0]['href'])
        self.assertEqual('instance', links[1]['rel'])
        self.assertTrue((self.PATH_PREFIX + '/meters/instance?'
                         'q.field=resource_id&q.value=resource-id')
                        in links[1]['href'])

    def test_resource_skip_meter_links(self):
        sample1 = sample.Sample(
            'instance',
            'cumulative',
            '',
            1,
            'user-id',
            'project-id',
            'resource-id',
            timestamp=datetime.datetime(2012, 7, 2, 10, 40),
            resource_metadata={'display_name': 'test-server',
                               'tag': 'self.sample',
                               },
            source='test_list_resources',
        )
        msg = utils.meter_message_from_counter(
            sample1, self.CONF.publisher.telemetry_secret,
        )
        self.conn.record_metering_data(msg)

        data = self.get_json('/resources?meter_links=0')
        links = data[0]['links']
        self.assertEqual(len(links), 1)
        self.assertEqual(links[0]['rel'], 'self')
        self.assertTrue((self.PATH_PREFIX + '/resources/resource-id')
                        in links[0]['href'])


class TestListResourcesRestriction(v2.FunctionalTest):
    def setUp(self):
        super(TestListResourcesRestriction, self).setUp()
        self.CONF.set_override('default_api_return_limit', 10, group='api')
        for i in range(20):
            s = sample.Sample(
                'volume.size',
                'gauge',
                'GiB',
                5 + i,
                'user-id',
                'project1',
                'resource-id%s' % i,
                timestamp=(datetime.datetime(2012, 9, 25, 10, 30) +
                           datetime.timedelta(seconds=i)),
                resource_metadata={'display_name': 'test-volume',
                                   'tag': 'self.sample',
                                   },
                source='source1',
            )
            msg = utils.meter_message_from_counter(
                s, self.CONF.publisher.telemetry_secret,
            )
            self.conn.record_metering_data(msg)

    def test_resource_limit(self):
        data = self.get_json('/resources?limit=1')
        self.assertEqual(1, len(data))

    def test_resource_limit_negative(self):
        self.assertRaises(webtest.app.AppError, self.get_json,
                          '/resources?limit=-2')

    def test_resource_limit_bigger(self):
        data = self.get_json('/resources?limit=42')
        self.assertEqual(20, len(data))

    def test_resource_default_limit(self):
        data = self.get_json('/resources')
        self.assertEqual(10, len(data))



#
# Copyright 2012 eNovance <licensing@enovance.com>
# Copyright 2012 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

from oslo_log import log

import ceilometer
from ceilometer.compute import pollsters
from ceilometer.compute.pollsters import util
from ceilometer.compute.virt import inspector as virt_inspector
from ceilometer.i18n import _
from ceilometer import sample

LOG = log.getLogger(__name__)


class _Base(pollsters.BaseComputePollster):

    NET_USAGE_MESSAGE = ' '.join(["NETWORK USAGE:", "%s %s:", "read-bytes=%d",
                                  "write-bytes=%d"])

    @staticmethod
    def make_vnic_sample(instance, name, type, unit, volume, vnic_data):
        metadata = copy.copy(vnic_data)
        additional_metadata = dict(zip(metadata._fields, metadata))
        if vnic_data.fref is not None:
            rid = vnic_data.fref
            additional_metadata['vnic_name'] = vnic_data.fref
        else:
            instance_name = util.instance_name(instance)
            rid = "%s-%s-%s" % (instance_name, instance.id, vnic_data.name)
            additional_metadata['vnic_name'] = vnic_data.name

        return util.make_sample_from_instance(
            instance=instance,
            name=name,
            type=type,
            unit=unit,
            volume=volume,
            resource_id=rid,
            additional_metadata=additional_metadata
        )

    CACHE_KEY_VNIC = 'vnics'

    def _get_vnic_info(self, inspector, instance):
        return inspector.inspect_vnics(instance)

    @staticmethod
    def _get_rx_info(info):
        return info.rx_bytes

    @staticmethod
    def _get_tx_info(info):
        return info.tx_bytes

    def _get_vnics_for_instance(self, cache, inspector, instance):
        i_cache = cache.setdefault(self.CACHE_KEY_VNIC, {})
        if instance.id not in i_cache:
            i_cache[instance.id] = list(
                self._get_vnic_info(inspector, instance)
            )
        return i_cache[instance.id]

    def get_samples(self, manager, cache, resources):
        self._inspection_duration = self._record_poll_time()
        for instance in resources:
            instance_name = util.instance_name(instance)
            LOG.debug('checking net info for instance %s', instance.id)
            try:
                vnics = self._get_vnics_for_instance(
                    cache,
                    self.inspector,
                    instance,
                )
                for vnic, info in vnics:
                    LOG.debug(self.NET_USAGE_MESSAGE, instance_name,
                              vnic.name, self._get_rx_info(info),
                              self._get_tx_info(info))
                    yield self._get_sample(instance, vnic, info)
            except virt_inspector.InstanceNotFoundException as err:
                # Instance was deleted while getting samples. Ignore it.
                LOG.debug('Exception while getting samples %s', err)
            except virt_inspector.InstanceShutOffException as e:
                LOG.debug('Instance %(instance_id)s was shut off while '
                          'getting samples of %(pollster)s: %(exc)s',
                          {'instance_id': instance.id,
                           'pollster': self.__class__.__name__, 'exc': e})
            except ceilometer.NotImplementedError:
                # Selected inspector does not implement this pollster.
                LOG.debug('%(inspector)s does not provide data for '
                          ' %(pollster)s',
                          {'inspector': self.inspector.__class__.__name__,
                           'pollster': self.__class__.__name__})
            except Exception as err:
                LOG.exception(_('Ignoring instance %(name)s: %(error)s'),
                              {'name': instance_name, 'error': err})


class _RateBase(_Base):

    NET_USAGE_MESSAGE = ' '.join(["NETWORK RATE:", "%s %s:",
                                  "read-bytes-rate=%d",
                                  "write-bytes-rate=%d"])

    CACHE_KEY_VNIC = 'vnic-rates'

    def _get_vnic_info(self, inspector, instance):
        return inspector.inspect_vnic_rates(instance,
                                            self._inspection_duration)

    @staticmethod
    def _get_rx_info(info):
        return info.rx_bytes_rate

    @staticmethod
    def _get_tx_info(info):
        return info.tx_bytes_rate


class IncomingBytesPollster(_Base):

    def _get_sample(self, instance, vnic, info):
        return self.make_vnic_sample(
            instance,
            name='network.incoming.bytes',
            type=sample.TYPE_CUMULATIVE,
            unit='B',
            volume=info.rx_bytes,
            vnic_data=vnic,
        )


class IncomingPacketsPollster(_Base):

    def _get_sample(self, instance, vnic, info):
        return self.make_vnic_sample(
            instance,
            name='network.incoming.packets',
            type=sample.TYPE_CUMULATIVE,
            unit='packet',
            volume=info.rx_packets,
            vnic_data=vnic,
        )


class OutgoingBytesPollster(_Base):

    def _get_sample(self, instance, vnic, info):
        return self.make_vnic_sample(
            instance,
            name='network.outgoing.bytes',
            type=sample.TYPE_CUMULATIVE,
            unit='B',
            volume=info.tx_bytes,
            vnic_data=vnic,
        )


class OutgoingPacketsPollster(_Base):

    def _get_sample(self, instance, vnic, info):
        return self.make_vnic_sample(
            instance,
            name='network.outgoing.packets',
            type=sample.TYPE_CUMULATIVE,
            unit='packet',
            volume=info.tx_packets,
            vnic_data=vnic,
        )


class IncomingBytesRatePollster(_RateBase):

    def _get_sample(self, instance, vnic, info):
        return self.make_vnic_sample(
            instance,
            name='network.incoming.bytes.rate',
            type=sample.TYPE_GAUGE,
            unit='B/s',
            volume=info.rx_bytes_rate,
            vnic_data=vnic,
        )


class OutgoingBytesRatePollster(_RateBase):

    def _get_sample(self, instance, vnic, info):
        return self.make_vnic_sample(
            instance,
            name='network.outgoing.bytes.rate',
            type=sample.TYPE_GAUGE,
            unit='B/s',
            volume=info.tx_bytes_rate,
            vnic_data=vnic,
        )



#
# Copyright Ericsson AB 2013. All rights reserved
#
# Authors: Ildiko Vancsa <ildiko.vancsa@ericsson.com>
#          Balazs Gibizer <balazs.gibizer@ericsson.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Tests complex queries for samples
"""

import datetime

from oslo_utils import timeutils

from ceilometer.publisher import utils
from ceilometer import sample
from ceilometer.tests.functional.api import v2 as tests_api


admin_header = {"X-Roles": "admin",
                "X-Project-Id":
                "project-id1"}
non_admin_header = {"X-Roles": "Member",
                    "X-Project-Id":
                    "project-id1"}


class TestQueryMetersController(tests_api.FunctionalTest):
    def setUp(self):
        super(TestQueryMetersController, self).setUp()
        self.url = '/query/samples'

        for cnt in [
            sample.Sample('meter.test',
                          'cumulative',
                          '',
                          1,
                          'user-id1',
                          'project-id1',
                          'resource-id1',
                          timestamp=datetime.datetime(2012, 7, 2, 10, 40),
                          resource_metadata={'display_name': 'test-server1',
                                             'tag': 'self.sample',
                                             'size': 456,
                                             'util': 0.25,
                                             'is_public': True},
                          source='test_source'),
            sample.Sample('meter.test',
                          'cumulative',
                          '',
                          2,
                          'user-id2',
                          'project-id2',
                          'resource-id2',
                          timestamp=datetime.datetime(2012, 7, 2, 10, 41),
                          resource_metadata={'display_name': 'test-server2',
                                             'tag': 'self.sample',
                                             'size': 123,
                                             'util': 0.75,
                                             'is_public': True},
                          source='test_source'),
            sample.Sample('meter.test',
                          'cumulative',
                          '',
                          3,
                          'user-id3',
                          'project-id3',
                          'resource-id3',
                          timestamp=datetime.datetime(2012, 7, 2, 10, 42),
                          resource_metadata={'display_name': 'test-server3',
                                             'tag': 'self.sample',
                                             'size': 789,
                                             'util': 0.95,
                                             'is_public': True},
                          source='test_source')]:

            msg = utils.meter_message_from_counter(
                cnt, self.CONF.publisher.telemetry_secret)
            self.conn.record_metering_data(msg)

    def test_query_fields_are_optional(self):
        data = self.post_json(self.url, params={})
        self.assertEqual(3, len(data.json))

    def test_query_with_isotime(self):
        date_time = datetime.datetime(2012, 7, 2, 10, 41)
        isotime = date_time.isoformat()

        data = self.post_json(self.url,
                              params={"filter":
                                      '{">=": {"timestamp": "'
                                      + isotime + '"}}'})

        self.assertEqual(2, len(data.json))
        for sample_item in data.json:
            result_time = timeutils.parse_isotime(sample_item['timestamp'])
            result_time = result_time.replace(tzinfo=None)
            self.assertTrue(result_time >= date_time)

    def test_non_admin_tenant_sees_only_its_own_project(self):
        data = self.post_json(self.url,
                              params={},
                              headers=non_admin_header)
        for sample_item in data.json:
            self.assertEqual("project-id1", sample_item['project_id'])

    def test_non_admin_tenant_cannot_query_others_project(self):
        data = self.post_json(self.url,
                              params={"filter":
                                      '{"=": {"project_id": "project-id2"}}'},
                              expect_errors=True,
                              headers=non_admin_header)

        self.assertEqual(401, data.status_int)
        self.assertIn(b"Not Authorized to access project project-id2",
                      data.body)

    def test_non_admin_tenant_can_explicitly_filter_for_own_project(self):
        data = self.post_json(self.url,
                              params={"filter":
                                      '{"=": {"project_id": "project-id1"}}'},
                              headers=non_admin_header)

        for sample_item in data.json:
            self.assertEqual("project-id1", sample_item['project_id'])

    def test_admin_tenant_sees_every_project(self):
        data = self.post_json(self.url,
                              params={},
                              headers=admin_header)

        self.assertEqual(3, len(data.json))
        for sample_item in data.json:
            self.assertIn(sample_item['project_id'],
                          (["project-id1", "project-id2", "project-id3"]))

    def test_admin_tenant_sees_every_project_with_complex_filter(self):
        filter = ('{"OR": ' +
                  '[{"=": {"project_id": "project-id1"}}, ' +
                  '{"=": {"project_id": "project-id2"}}]}')
        data = self.post_json(self.url,
                              params={"filter": filter},
                              headers=admin_header)

        self.assertEqual(2, len(data.json))
        for sample_item in data.json:
            self.assertIn(sample_item['project_id'],
                          (["project-id1", "project-id2"]))

    def test_admin_tenant_sees_every_project_with_in_filter(self):
        filter = ('{"In": ' +
                  '{"project_id": ["project-id1", "project-id2"]}}')
        data = self.post_json(self.url,
                              params={"filter": filter},
                              headers=admin_header)

        self.assertEqual(2, len(data.json))
        for sample_item in data.json:
            self.assertIn(sample_item['project_id'],
                          (["project-id1", "project-id2"]))

    def test_admin_tenant_can_query_any_project(self):
        data = self.post_json(self.url,
                              params={"filter":
                                      '{"=": {"project_id": "project-id2"}}'},
                              headers=admin_header)

        self.assertEqual(1, len(data.json))
        for sample_item in data.json:
            self.assertIn(sample_item['project_id'], set(["project-id2"]))

    def test_query_with_orderby(self):
        data = self.post_json(self.url,
                              params={"orderby": '[{"project_id": "DESC"}]'})

        self.assertEqual(3, len(data.json))
        self.assertEqual(["project-id3", "project-id2", "project-id1"],
                         [s["project_id"] for s in data.json])

    def test_query_with_field_name_project(self):
        data = self.post_json(self.url,
                              params={"filter":
                                      '{"=": {"project": "project-id2"}}'})

        self.assertEqual(1, len(data.json))
        for sample_item in data.json:
            self.assertIn(sample_item['project_id'], set(["project-id2"]))

    def test_query_with_field_name_resource(self):
        data = self.post_json(self.url,
                              params={"filter":
                                      '{"=": {"resource": "resource-id2"}}'})

        self.assertEqual(1, len(data.json))
        for sample_item in data.json:
            self.assertIn(sample_item['resource_id'], set(["resource-id2"]))

    def test_query_with_wrong_field_name(self):
        data = self.post_json(self.url,
                              params={"filter":
                                      '{"=": {"unknown": "resource-id2"}}'},
                              expect_errors=True)

        self.assertEqual(400, data.status_int)
        self.assertIn(b"is not valid under any of the given schemas",
                      data.body)

    def test_query_with_wrong_json(self):
        data = self.post_json(self.url,
                              params={"filter":
                                      '{"=": "resource": "resource-id2"}}'},
                              expect_errors=True)

        self.assertEqual(400, data.status_int)
        self.assertIn(b"Filter expression not valid", data.body)

    def test_query_with_field_name_user(self):
        data = self.post_json(self.url,
                              params={"filter":
                                      '{"=": {"user": "user-id2"}}'})

        self.assertEqual(1, len(data.json))
        for sample_item in data.json:
            self.assertIn(sample_item['user_id'], set(["user-id2"]))

    def test_query_with_field_name_meter(self):
        data = self.post_json(self.url,
                              params={"filter":
                                      '{"=": {"meter": "meter.test"}}'})

        self.assertEqual(3, len(data.json))
        for sample_item in data.json:
            self.assertIn(sample_item['meter'], set(["meter.test"]))

    def test_query_with_lower_and_upper_case_orderby(self):
        data = self.post_json(self.url,
                              params={"orderby": '[{"project_id": "DeSc"}]'})

        self.assertEqual(3, len(data.json))
        self.assertEqual(["project-id3", "project-id2", "project-id1"],
                         [s["project_id"] for s in data.json])

    def test_query_with_user_field_name_orderby(self):
        data = self.post_json(self.url,
                              params={"orderby": '[{"user": "aSc"}]'})

        self.assertEqual(3, len(data.json))
        self.assertEqual(["user-id1", "user-id2", "user-id3"],
                         [s["user_id"] for s in data.json])

    def test_query_with_volume_field_name_orderby(self):
        data = self.post_json(self.url,
                              params={"orderby": '[{"volume": "deSc"}]'})

        self.assertEqual(3, len(data.json))
        self.assertEqual([3, 2, 1],
                         [s["volume"] for s in data.json])

    def test_query_with_missing_order_in_orderby(self):
        data = self.post_json(self.url,
                              params={"orderby": '[{"project_id": ""}]'},
                              expect_errors=True)

        self.assertEqual(400, data.status_int)
        self.assertIn(b"does not match '(?i)^asc$|^desc$'", data.body)

    def test_query_with_wrong_json_in_orderby(self):
        data = self.post_json(self.url,
                              params={"orderby": '{"project_id": "desc"}]'},
                              expect_errors=True)

        self.assertEqual(400, data.status_int)
        self.assertIn(b"Order-by expression not valid: Extra data", data.body)

    def test_filter_with_metadata(self):
        data = self.post_json(self.url,
                              params={"filter":
                                      '{">=": {"metadata.util": 0.5}}'})

        self.assertEqual(2, len(data.json))
        for sample_item in data.json:
            self.assertTrue(float(sample_item["metadata"]["util"]) >= 0.5)

    def test_filter_with_negation(self):
        filter_expr = '{"not": {">=": {"metadata.util": 0.5}}}'
        data = self.post_json(self.url,
                              params={"filter": filter_expr})

        self.assertEqual(1, len(data.json))
        for sample_item in data.json:
            self.assertTrue(float(sample_item["metadata"]["util"]) < 0.5)

    def test_limit_must_be_positive(self):
        data = self.post_json(self.url,
                              params={"limit": 0},
                              expect_errors=True)

        self.assertEqual(400, data.status_int)
        self.assertIn(b"Limit must be positive", data.body)

    def test_default_limit(self):
        self.CONF.set_override('default_api_return_limit', 1, group='api')
        data = self.post_json(self.url, params={})
        self.assertEqual(1, len(data.json))



#
# Copyright 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from migrate import ForeignKeyConstraint
from sqlalchemy import MetaData, Table, Column, Index
from sqlalchemy import String, DateTime


def upgrade(migrate_engine):
    meta = MetaData()
    meta.bind = migrate_engine

    project = Table('project', meta, autoload=True)
    user = Table('user', meta, autoload=True)

    alarm_history = Table(
        'alarm_history', meta,
        Column('event_id', String(255), primary_key=True, index=True),
        Column('alarm_id', String(255)),
        Column('on_behalf_of', String(255)),
        Column('project_id', String(255)),
        Column('user_id', String(255)),
        Column('type', String(20)),
        Column('detail', String(255)),
        Column('timestamp', DateTime(timezone=False)),
        mysql_engine='InnoDB',
        mysql_charset='utf8')

    alarm_history.create()

    if migrate_engine.name in ['mysql', 'postgresql']:
        indices = [Index('ix_alarm_history_alarm_id',
                         alarm_history.c.alarm_id),
                   Index('ix_alarm_history_on_behalf_of',
                         alarm_history.c.on_behalf_of),
                   Index('ix_alarm_history_project_id',
                         alarm_history.c.project_id),
                   Index('ix_alarm_history_on_user_id',
                         alarm_history.c.user_id)]

        for index in indices:
            index.create(migrate_engine)

        fkeys = [ForeignKeyConstraint(columns=[alarm_history.c.on_behalf_of],
                                      refcolumns=[project.c.id]),
                 ForeignKeyConstraint(columns=[alarm_history.c.project_id],
                                      refcolumns=[project.c.id]),
                 ForeignKeyConstraint(columns=[alarm_history.c.user_id],
                                      refcolumns=[user.c.id])]
        for fkey in fkeys:
            fkey.create(engine=migrate_engine)



#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from sqlalchemy import Index, MetaData, Table


INDEXES = {
    # `table_name`: ((`index_name`, `column`),)
    "user": (('ix_user_id', 'id'),),
    "source": (('ix_source_id', 'id'),),
    "project": (('ix_project_id', 'id'),),
    "meter": (('ix_meter_id', 'id'),),
    "alarm": (('ix_alarm_id', 'id'),),
    "resource": (('ix_resource_id', 'id'),)
}


def upgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    load_tables = dict((table_name, Table(table_name, meta, autoload=True))
                       for table_name in INDEXES.keys())
    for table_name, indexes in INDEXES.items():
        table = load_tables[table_name]
        for index_name, column in indexes:
            index = Index(index_name, table.c[column])
            index.drop()



#
# Copyright 2014 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
import six


# Below config is for collecting metadata which user defined in nova or else,
# and then storing it to Sample for future use according to user's requirement.
# Such as using it as OpenTSDB tags for metrics.
OPTS = [
    cfg.ListOpt('reserved_metadata_namespace',
                default=['metering.'],
                help='List of metadata prefixes reserved for metering use.'),
    cfg.IntOpt('reserved_metadata_length',
               default=256,
               help='Limit on length of reserved metadata values.'),
    cfg.ListOpt('reserved_metadata_keys',
                default=[],
                help='List of metadata keys reserved for metering use. And '
                     'these keys are additional to the ones included in the '
                     'namespace.'),
]

cfg.CONF.register_opts(OPTS)


def add_reserved_user_metadata(src_metadata, dest_metadata):
    limit = cfg.CONF.reserved_metadata_length
    user_metadata = {}
    for prefix in cfg.CONF.reserved_metadata_namespace:
        md = dict(
            (k[len(prefix):].replace('.', '_'),
             v[:limit] if isinstance(v, six.string_types) else v)
            for k, v in src_metadata.items()
            if (k.startswith(prefix) and
                k[len(prefix):].replace('.', '_') not in dest_metadata)
        )
        user_metadata.update(md)

    for metadata_key in cfg.CONF.reserved_metadata_keys:
        md = dict(
            (k.replace('.', '_'),
             v[:limit] if isinstance(v, six.string_types) else v)
            for k, v in src_metadata.items()
            if (k == metadata_key and
                k.replace('.', '_') not in dest_metadata)
        )
        user_metadata.update(md)

    if user_metadata:
        dest_metadata['user_metadata'] = user_metadata

    return dest_metadata



# Copyright 2014 Intel Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Node manager engine to collect power and temperature of compute node.

Intel Node Manager Technology enables the datacenter IT to monitor and control
actual server power, thermal and compute utlization behavior through industry
defined standard IPMI. This file provides Node Manager engine to get simple
system power and temperature data based on ipmitool.
"""

import binascii
import collections
import tempfile
import time

from oslo_config import cfg
import six

from ceilometer.i18n import _
from ceilometer.ipmi.platform import exception as nmexcept
from ceilometer.ipmi.platform import ipmitool


OPTS = [
    cfg.IntOpt('node_manager_init_retry',
               default=3,
               help='Number of retries upon Intel Node '
                    'Manager initialization failure')
]


CONF = cfg.CONF
CONF.register_opts(OPTS, group='ipmi')

IPMICMD = {"sdr_dump": "sdr dump",
           "sdr_info": "sdr info",
           "sensor_dump": "sdr -v"}
IPMIRAWCMD = {"get_device_id": "raw 0x06 0x01",
              "get_nm_version": "raw 0x2e 0xca 0x57 0x01 0x00",
              "init_sensor_agent": "raw 0x0a 0x2c 0x01",
              "init_complete": "raw 0x0a 0x2c 0x00",
              "init_sensor_agent_status": "raw 0x0a 0x2c 0x00",
              "read_power_all": "raw 0x2e 0xc8 0x57 0x01 0x00 0x01 0x00 0x00",
              "read_inlet_temperature":
              "raw 0x2e 0xc8 0x57 0x01 0x00 0x02 0x00 0x00",
              "read_outlet_temperature":
              "raw 0x2e 0xc8 0x57 0x01 0x00 0x05 0x00 0x00",
              "read_airflow": "raw 0x2e 0xc8 0x57 0x01 0x00 0x04 0x00 0x00",
              "read_cups_utilization": "raw 0x2e 0x65 0x57 0x01 0x00 0x05",
              "read_cups_index": "raw 0x2e 0x65 0x57 0x01 0x00 0x01"}

MANUFACTURER_ID_INTEL = ['57', '01', '00']
INTEL_PREFIX = '5701000d01'

# The template dict are made according to the spec. It contains the expected
# length of each item. And it can be used to parse the output of IPMI command.

ONE_RETURN_TEMPLATE = {"ret": 1}

BMC_INFO_TEMPLATE = collections.OrderedDict()
BMC_INFO_TEMPLATE['Device_ID'] = 1
BMC_INFO_TEMPLATE['Device_Revision'] = 1
BMC_INFO_TEMPLATE['Firmware_Revision_1'] = 1
BMC_INFO_TEMPLATE['Firmware_Revision_2'] = 1
BMC_INFO_TEMPLATE['IPMI_Version'] = 1
BMC_INFO_TEMPLATE['Additional_Device_support'] = 1
BMC_INFO_TEMPLATE['Manufacturer_ID'] = 3
BMC_INFO_TEMPLATE['Product_ID'] = 2
BMC_INFO_TEMPLATE['Auxiliary_Firmware_Revision'] = 4

NM_STATISTICS_TEMPLATE = collections.OrderedDict()
NM_STATISTICS_TEMPLATE['Manufacturer_ID'] = 3
NM_STATISTICS_TEMPLATE['Current_value'] = 2
NM_STATISTICS_TEMPLATE['Minimum_value'] = 2
NM_STATISTICS_TEMPLATE['Maximum_value'] = 2
NM_STATISTICS_TEMPLATE['Average_value'] = 2
NM_STATISTICS_TEMPLATE['Time_stamp'] = 4
NM_STATISTICS_TEMPLATE['Report_period'] = 4
NM_STATISTICS_TEMPLATE["DomainID_PolicyState"] = 1

NM_GET_DEVICE_ID_TEMPLATE = collections.OrderedDict()
NM_GET_DEVICE_ID_TEMPLATE['Device_ID'] = 1
NM_GET_DEVICE_ID_TEMPLATE['Device_revision'] = 1
NM_GET_DEVICE_ID_TEMPLATE['Firmware_revision_1'] = 1
NM_GET_DEVICE_ID_TEMPLATE['Firmware_Revision_2'] = 1
NM_GET_DEVICE_ID_TEMPLATE['IPMI_Version'] = 1
NM_GET_DEVICE_ID_TEMPLATE['Additinal_Device_support'] = 1
NM_GET_DEVICE_ID_TEMPLATE['Manufacturer_ID'] = 3
NM_GET_DEVICE_ID_TEMPLATE['Product_ID_min_version'] = 1
NM_GET_DEVICE_ID_TEMPLATE['Product_ID_major_version'] = 1
NM_GET_DEVICE_ID_TEMPLATE['Implemented_firmware'] = 1
NM_GET_DEVICE_ID_TEMPLATE['Firmware_build_number'] = 1
NM_GET_DEVICE_ID_TEMPLATE['Last_digit_firmware_build_number'] = 1
NM_GET_DEVICE_ID_TEMPLATE['Image_flags'] = 1

NM_GET_VERSION_TEMPLATE = collections.OrderedDict()
NM_GET_VERSION_TEMPLATE['Manufacturer_ID'] = 3
NM_GET_VERSION_TEMPLATE['NM_Version'] = 1
NM_GET_VERSION_TEMPLATE['IPMI_Version'] = 1
NM_GET_VERSION_TEMPLATE['Patch_Version'] = 1
NM_GET_VERSION_TEMPLATE['Firmware_Revision_Major'] = 1
NM_GET_VERSION_TEMPLATE['Firmware_Revision_Minor'] = 1

NM_CUPS_UTILIZATION_TEMPLATE = collections.OrderedDict()
NM_CUPS_UTILIZATION_TEMPLATE['Manufacturer_ID'] = 3
NM_CUPS_UTILIZATION_TEMPLATE['CPU_Utilization'] = 8
NM_CUPS_UTILIZATION_TEMPLATE['Mem_Utilization'] = 8
NM_CUPS_UTILIZATION_TEMPLATE['IO_Utilization'] = 8

NM_CUPS_INDEX_TEMPLATE = collections.OrderedDict()
NM_CUPS_INDEX_TEMPLATE['Manufacturer_ID'] = 3
NM_CUPS_INDEX_TEMPLATE['CUPS_Index'] = 2


def _hex(list=None):
    """Format the return value in list into hex."""

    list = list or []
    if list:
        list.reverse()
        return int(''.join(list), 16)

    return 0


class NodeManager(object):
    """The python implementation of Intel Node Manager engine using ipmitool

    The class implements the engine to read power and temperature of
    compute node. It uses ipmitool to execute the IPMI command and parse
    the output into dict.
    """
    _inited = False
    _instance = None

    def __new__(cls, *args, **kwargs):
        """Singleton to avoid duplicated initialization."""
        if not cls._instance:
            cls._instance = super(NodeManager, cls).__new__(cls, *args,
                                                            **kwargs)
        return cls._instance

    def __init__(self):
        if not (self._instance and self._inited):
            # As singleton, only the 1st NM pollster would trigger its
            # initialization. nm_version indicate init result, and is shared
            # across all pollsters
            self._inited = True
            self.nm_version = 0
            self.channel_slave = ''

            self.nm_version = self.check_node_manager()

    @staticmethod
    def _parse_slave_and_channel(file_path):
        """Parse the dumped file to get slave address and channel number.

        :param file_path: file path of dumped SDR file.
        :return: slave address and channel number of target device.
        """
        ret = None
        prefix = INTEL_PREFIX
        # According to Intel Node Manager spec, section 4.5, for Intel NM
        # discovery OEM SDR records are type C0h. It contains manufacture ID
        # and OEM data in the record body.
        # 0-2 bytes are OEM ID, byte 3 is 0Dh and byte 4 is 01h. Byte 5, 6
        # is Intel NM device slave address and channel number/sensor owner LUN.
        with open(file_path, 'rb') as bin_fp:
            for line in bin_fp.readlines():
                if line:
                    data_str = binascii.hexlify(line)
                    if six.PY3:
                        data_str = data_str.decode('ascii')
                    if prefix in data_str:
                        oem_id_index = data_str.index(prefix)
                        ret = data_str[oem_id_index + len(prefix):
                                       oem_id_index + len(prefix) + 4]
                        # Byte 5 is slave address. [7:4] from byte 6 is channel
                        # number, so just pick ret[2] here.
                        ret = (ret[0:2], ret[2])
                        break
        return ret

    @ipmitool.execute_ipmi_cmd(BMC_INFO_TEMPLATE)
    def get_device_id(self):
        """IPMI command GET_DEVICE_ID."""
        return IPMIRAWCMD["get_device_id"]

    @ipmitool.execute_ipmi_cmd(ONE_RETURN_TEMPLATE)
    def _init_sensor_agent(self):
        """Run initialization agent."""
        return IPMIRAWCMD["init_sensor_agent"]

    @ipmitool.execute_ipmi_cmd(ONE_RETURN_TEMPLATE)
    def _init_sensor_agent_process(self):
        """Check the status of initialization agent."""
        return IPMIRAWCMD["init_sensor_agent_status"]

    @ipmitool.execute_ipmi_cmd()
    def _dump_sdr_file(self, data_file=""):
        """Dump SDR into a file."""
        return IPMICMD["sdr_dump"] + " " + data_file

    @ipmitool.execute_ipmi_cmd(NM_GET_DEVICE_ID_TEMPLATE)
    def _node_manager_get_device_id(self):
        """GET_DEVICE_ID command in Intel Node Manager

        Different from IPMI command GET_DEVICE_ID, it contains more information
        of Intel Node Manager.
        """
        return self.channel_slave + ' ' + IPMIRAWCMD["get_device_id"]

    @ipmitool.execute_ipmi_cmd(NM_GET_VERSION_TEMPLATE)
    def _node_manager_get_version(self):
        """GET_NODE_MANAGER_VERSION command in Intel Node Manager

        Byte 4 of the response:
        01h - Intel NM 1.0
        02h - Intel NM 1.5
        03h - Intel NM 2.0
        04h - Intel NM 2.5
        05h - Intel NM 3.0
        """
        return self.channel_slave + ' ' + IPMIRAWCMD["get_nm_version"]

    @ipmitool.execute_ipmi_cmd(NM_STATISTICS_TEMPLATE)
    def _read_power_all(self):
        """Get the power consumption of the whole platform."""
        return self.channel_slave + ' ' + IPMIRAWCMD['read_power_all']

    @ipmitool.execute_ipmi_cmd(NM_STATISTICS_TEMPLATE)
    def _read_inlet_temperature(self):
        """Get the inlet temperature info of the whole platform."""
        return self.channel_slave + ' ' + IPMIRAWCMD['read_inlet_temperature']

    @ipmitool.execute_ipmi_cmd(NM_STATISTICS_TEMPLATE)
    def _read_outlet_temperature(self):
        """Get the outlet temperature info of the whole platform."""
        return self.channel_slave + ' ' + IPMIRAWCMD['read_outlet_temperature']

    @ipmitool.execute_ipmi_cmd(NM_STATISTICS_TEMPLATE)
    def _read_airflow(self):
        """Get the volumetric airflow of the whole platform."""
        return self.channel_slave + ' ' + IPMIRAWCMD['read_airflow']

    @ipmitool.execute_ipmi_cmd(NM_CUPS_UTILIZATION_TEMPLATE)
    def _read_cups_utilization(self):
        """Get the average CUPS utilization of the whole platform."""
        return self.channel_slave + ' ' + IPMIRAWCMD['read_cups_utilization']

    @ipmitool.execute_ipmi_cmd(NM_CUPS_INDEX_TEMPLATE)
    def _read_cups_index(self):
        """Get the CUPS Index of the whole platform."""
        return self.channel_slave + ' ' + IPMIRAWCMD['read_cups_index']

    def read_power_all(self):
        return self._read_power_all() if self.nm_version > 0 else {}

    def read_inlet_temperature(self):
        return self._read_inlet_temperature() if self.nm_version > 0 else {}

    def read_outlet_temperature(self):
        return self._read_outlet_temperature() if self.nm_version >= 5 else {}

    def read_airflow(self):
        # only available after NM 3.0
        return self._read_airflow() if self.nm_version >= 5 else {}

    def read_cups_utilization(self):
        # only available after NM 3.0
        return self._read_cups_utilization() if self.nm_version >= 5 else {}

    def read_cups_index(self):
        # only available after NM 3.0
        return self._read_cups_index() if self.nm_version >= 5 else {}

    def init_node_manager(self):
        if self._init_sensor_agent_process()['ret'] == ['01']:
            return
        # Run sensor initialization agent
        for i in range(CONF.ipmi.node_manager_init_retry):
            self._init_sensor_agent()
            time.sleep(1)
            if self._init_sensor_agent_process()['ret'] == ['01']:
                return

        raise nmexcept.NodeManagerException(_('Node Manager init failed'))

    def discover_slave_channel(self):
        """Discover target slave address and channel number."""
        file_path = tempfile.mkstemp()[1]
        self._dump_sdr_file(data_file=file_path)
        ret = self._parse_slave_and_channel(file_path)
        slave_address = ''.join(['0x', ret[0]])
        channel = ''.join(['0x', ret[1]])
        # String of channel and slave_address
        self.channel_slave = '-b ' + channel + ' -t ' + slave_address

    def node_manager_version(self):
        """Intel Node Manager capability checking

        This function is used to detect if compute node support Intel Node
        Manager(return version number) or not(return -1) and parse out the
        slave address and channel number of node manager.
        """
        self.manufacturer_id = self.get_device_id()['Manufacturer_ID']
        if MANUFACTURER_ID_INTEL != self.manufacturer_id:
            # If the manufacturer is not Intel, just set False and return.
            return 0

        self.discover_slave_channel()
        support = self._node_manager_get_device_id()['Implemented_firmware']
        # According to Intel Node Manager spec, return value of GET_DEVICE_ID,
        # bits 3 to 0 shows if Intel NM implemented or not.
        if int(support[0], 16) & 0xf == 0:
            return 0

        return _hex(self._node_manager_get_version()['NM_Version'])

    def check_node_manager(self):
        """Intel Node Manager init and check

        This function is used to initialize Intel Node Manager and check the
        capability without throwing exception. It's safe to call it on
        non-NodeManager platform.
        """
        try:
            self.init_node_manager()
            nm_version = self.node_manager_version()
        except (nmexcept.NodeManagerException, nmexcept.IPMIException):
            return 0
        return nm_version



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from migrate import ForeignKeyConstraint
from sqlalchemy import Column
from sqlalchemy import Integer
from sqlalchemy import MetaData
from sqlalchemy import select
from sqlalchemy import String
from sqlalchemy import Table

from ceilometer.storage.sqlalchemy import migration


def upgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    event_type = Table(
        'event_type', meta,
        Column('id', Integer, primary_key=True),
        Column('desc', String(255), unique=True),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )
    event_type.create()
    event = Table('event', meta, autoload=True)
    unique_name = Table('unique_name', meta, autoload=True)

    # Event type is a specialization of Unique name, so
    # we insert into the event_type table all the distinct
    # unique names from the event.unique_name field along
    # with the key from the unique_name table, and
    # then rename the event.unique_name field to event.event_type
    conn = migrate_engine.connect()
    sql = ("INSERT INTO event_type "
           "SELECT unique_name.id, unique_name.key FROM event "
           "INNER JOIN unique_name "
           "ON event.unique_name_id = unique_name.id "
           "GROUP BY unique_name.id")
    conn.execute(sql)
    conn.close()
    # Now we need to drop the foreign key constraint, rename
    # the event.unique_name column, and re-add a new foreign
    # key constraint
    params = {'columns': [event.c.unique_name_id],
              'refcolumns': [unique_name.c.id]}
    if migrate_engine.name == 'mysql':
        params['name'] = "event_ibfk_1"
    fkey = ForeignKeyConstraint(**params)
    fkey.drop()

    Column('event_type_id', Integer).create(event)

    # Move data from unique_name_id column into event_type_id column
    # and delete the entry from the unique_name table
    query = select([event.c.id, event.c.unique_name_id])
    for key, value in migration.paged(query):
        (event.update().where(event.c.id == key).
         values({"event_type_id": value}).execute())
        unique_name.delete().where(unique_name.c.id == key).execute()

    params = {'columns': [event.c.event_type_id],
              'refcolumns': [event_type.c.id]}
    if migrate_engine.name == 'mysql':
        params['name'] = "_".join(('fk', 'event_type', 'id'))
    fkey = ForeignKeyConstraint(**params)
    fkey.create()

    event.c.unique_name_id.drop()



# Copyright 2015 Reliance Jio Infocomm Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections

from keystoneauth1 import exceptions
import mock
from oslotest import base
from oslotest import mockpatch
import testscenarios.testcase

from ceilometer.agent import manager
from ceilometer.objectstore import rgw
from ceilometer.objectstore.rgw_client import RGWAdminClient as rgw_client

bucket_list1 = [rgw_client.Bucket('somefoo1', 10, 7)]
bucket_list2 = [rgw_client.Bucket('somefoo2', 2, 9)]
bucket_list3 = [rgw_client.Bucket('unlisted', 100, 100)]

GET_BUCKETS = [('tenant-000', {'num_buckets': 2, 'size': 1042,
                               'num_objects': 1001, 'buckets': bucket_list1}),
               ('tenant-001', {'num_buckets': 2, 'size': 1042,
                               'num_objects': 1001, 'buckets': bucket_list2}),
               ('tenant-002-ignored', {'num_buckets': 2, 'size': 1042,
                                       'num_objects': 1001,
                                       'buckets': bucket_list3})]

GET_USAGE = [('tenant-000', 10),
             ('tenant-001', 11),
             ('tenant-002-ignored', 12)]

Tenant = collections.namedtuple('Tenant', 'id')
ASSIGNED_TENANTS = [Tenant('tenant-000'), Tenant('tenant-001')]


class TestManager(manager.AgentManager):

    def __init__(self):
        super(TestManager, self).__init__()
        self._keystone = mock.Mock()
        self._catalog = (self._keystone.session.auth.get_access.
                         return_value.service_catalog)
        self._catalog.url_for.return_value = 'http://foobar/endpoint'


class TestRgwPollster(testscenarios.testcase.WithScenarios,
                      base.BaseTestCase):

    # Define scenarios to run all of the tests against all of the
    # pollsters.
    scenarios = [
        ('radosgw.objects',
         {'factory': rgw.ObjectsPollster}),
        ('radosgw.objects.size',
         {'factory': rgw.ObjectsSizePollster}),
        ('radosgw.objects.containers',
         {'factory': rgw.ObjectsContainersPollster}),
        ('radosgw.containers.objects',
         {'factory': rgw.ContainersObjectsPollster}),
        ('radosgw.containers.objects.size',
         {'factory': rgw.ContainersSizePollster}),
        ('radosgw.api.request',
         {'factory': rgw.UsagePollster}),
    ]

    @staticmethod
    def fake_ks_service_catalog_url_for(*args, **kwargs):
        raise exceptions.EndpointNotFound("Fake keystone exception")

    def fake_iter_accounts(self, ksclient, cache, tenants):
        tenant_ids = [t.id for t in tenants]
        for i in self.ACCOUNTS:
            if i[0] in tenant_ids:
                yield i

    @mock.patch('ceilometer.pipeline.setup_pipeline', mock.MagicMock())
    def setUp(self):
        super(TestRgwPollster, self).setUp()
        self.pollster = self.factory()
        self.manager = TestManager()

        if self.pollster.CACHE_KEY_METHOD == 'rgw.get_bucket':
            self.ACCOUNTS = GET_BUCKETS
        else:
            self.ACCOUNTS = GET_USAGE

    def tearDown(self):
        super(TestRgwPollster, self).tearDown()
        rgw._Base._ENDPOINT = None

    def test_iter_accounts_no_cache(self):
        cache = {}
        with mockpatch.PatchObject(self.factory, '_get_account_info',
                                   return_value=[]):
            data = list(self.pollster._iter_accounts(mock.Mock(), cache,
                                                     ASSIGNED_TENANTS))

        self.assertIn(self.pollster.CACHE_KEY_METHOD, cache)
        self.assertEqual([], data)

    def test_iter_accounts_cached(self):
        # Verify that if a method has already been called, _iter_accounts
        # uses the cached version and doesn't call rgw_clinet.
        mock_method = mock.Mock()
        mock_method.side_effect = AssertionError(
            'should not be called',
        )

        api_method = 'get_%s' % self.pollster.METHOD

        with mockpatch.PatchObject(rgw_client, api_method, new=mock_method):
            cache = {self.pollster.CACHE_KEY_METHOD: [self.ACCOUNTS[0]]}
            data = list(self.pollster._iter_accounts(mock.Mock(), cache,
                                                     ASSIGNED_TENANTS))
        self.assertEqual([self.ACCOUNTS[0]], data)

    def test_metering(self):
        with mockpatch.PatchObject(self.factory, '_iter_accounts',
                                   side_effect=self.fake_iter_accounts):
            samples = list(self.pollster.get_samples(self.manager, {},
                                                     ASSIGNED_TENANTS))

        self.assertEqual(2, len(samples), self.pollster.__class__)

    def test_get_meter_names(self):
        with mockpatch.PatchObject(self.factory, '_iter_accounts',
                                   side_effect=self.fake_iter_accounts):
            samples = list(self.pollster.get_samples(self.manager, {},
                                                     ASSIGNED_TENANTS))

        self.assertEqual(set([samples[0].name]),
                         set([s.name for s in samples]))

    def test_only_poll_assigned(self):
        mock_method = mock.MagicMock()
        endpoint = 'http://127.0.0.1:8000/admin'
        api_method = 'get_%s' % self.pollster.METHOD
        with mockpatch.PatchObject(rgw_client, api_method, new=mock_method):
            with mockpatch.PatchObject(
                    self.manager._catalog, 'url_for',
                    return_value=endpoint):
                list(self.pollster.get_samples(self.manager, {},
                                               ASSIGNED_TENANTS))
        expected = [mock.call(t.id)
                    for t in ASSIGNED_TENANTS]
        self.assertEqual(expected, mock_method.call_args_list)

    def test_get_endpoint_only_once(self):
        mock_url_for = mock.MagicMock()
        mock_url_for.return_value = '/endpoint'
        api_method = 'get_%s' % self.pollster.METHOD
        with mockpatch.PatchObject(rgw_client, api_method,
                                   new=mock.MagicMock()):
            with mockpatch.PatchObject(
                    self.manager._catalog, 'url_for',
                    new=mock_url_for):
                list(self.pollster.get_samples(self.manager, {},
                                               ASSIGNED_TENANTS))
                list(self.pollster.get_samples(self.manager, {},
                                               ASSIGNED_TENANTS))
        self.assertEqual(1, mock_url_for.call_count)

    def test_endpoint_notfound(self):
        with mockpatch.PatchObject(
                self.manager._catalog, 'url_for',
                side_effect=self.fake_ks_service_catalog_url_for):
            samples = list(self.pollster.get_samples(self.manager, {},
                                                     ASSIGNED_TENANTS))

        self.assertEqual(0, len(samples))



#
# Copyright 2016 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from oslo_context import context
from oslo_utils import timeutils
from oslotest import base

from ceilometer import sample
from ceilometer.transformer import conversions


class AggregatorTransformerTestCase(base.BaseTestCase):
    SAMPLE = sample.Sample(
        name='cpu',
        type=sample.TYPE_CUMULATIVE,
        unit='ns',
        volume='1234567',
        user_id='56c5692032f34041900342503fecab30',
        project_id='ac9494df2d9d4e709bac378cceabaf23',
        resource_id='1ca738a1-c49c-4401-8346-5c60ebdb03f4',
        timestamp="2015-10-29 14:12:15.485877+00:00",
        resource_metadata={}
    )

    def setUp(self):
        super(AggregatorTransformerTestCase, self).setUp()
        self._sample_offset = 0

    def test_init_input_validation(self):
        aggregator = conversions.AggregatorTransformer("2", "15", None,
                                                       None, None)
        self.assertEqual(2, aggregator.size)
        self.assertEqual(15, aggregator.retention_time)

    def test_init_no_size_or_rention_time(self):
        aggregator = conversions.AggregatorTransformer()
        self.assertEqual(1, aggregator.size)
        self.assertIsNone(aggregator.retention_time)

    def test_init_size_zero(self):
        aggregator = conversions.AggregatorTransformer(size="0")
        self.assertEqual(1, aggregator.size)
        self.assertIsNone(aggregator.retention_time)

    def test_init_input_validation_size_invalid(self):
        self.assertRaises(ValueError, conversions.AggregatorTransformer,
                          "abc", "15", None, None, None)

    def test_init_input_validation_retention_time_invalid(self):
        self.assertRaises(ValueError, conversions.AggregatorTransformer,
                          "2", "abc", None, None, None)

    def test_init_no_timestamp(self):
        aggregator = conversions.AggregatorTransformer("1", "1", None,
                                                       None, None)
        self.assertEqual("first", aggregator.timestamp)

    def test_init_timestamp_none(self):
        aggregator = conversions.AggregatorTransformer("1", "1", None,
                                                       None, None, None)
        self.assertEqual("first", aggregator.timestamp)

    def test_init_timestamp_first(self):
        aggregator = conversions.AggregatorTransformer("1", "1", None,
                                                       None, None, "first")
        self.assertEqual("first", aggregator.timestamp)

    def test_init_timestamp_last(self):
        aggregator = conversions.AggregatorTransformer("1", "1", None,
                                                       None, None, "last")
        self.assertEqual("last", aggregator.timestamp)

    def test_init_timestamp_invalid(self):
        aggregator = conversions.AggregatorTransformer("1", "1", None,
                                                       None, None,
                                                       "invalid_option")
        self.assertEqual("first", aggregator.timestamp)

    def test_size_unbounded(self):
        aggregator = conversions.AggregatorTransformer(size="0",
                                                       retention_time="300")
        self._insert_sample_data(aggregator)

        samples = aggregator.flush(context.get_admin_context())

        self.assertEqual([], samples)

    def test_size_bounded(self):
        aggregator = conversions.AggregatorTransformer(size="100")
        self._insert_sample_data(aggregator)

        samples = aggregator.flush(context.get_admin_context())

        self.assertEqual(100, len(samples))

    def _insert_sample_data(self, aggregator):
        for _ in range(100):
            sample = copy.copy(self.SAMPLE)
            sample.resource_id = sample.resource_id + str(self._sample_offset)
            sample.timestamp = timeutils.isotime()
            aggregator.handle_sample(context.get_admin_context(), sample)
            self._sample_offset += 1



#
# Copyright 2012 New Dream Network, LLC (DreamHost)
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Tests for ceilometer.network.notifications
"""

import mock

from ceilometer.network import notifications
from ceilometer.tests import base as test

NOTIFICATION_NETWORK_CREATE = {
    u'_context_roles': [u'anotherrole',
                        u'Member'],
    u'_context_read_deleted': u'no',
    u'event_type': u'network.create.end',
    u'timestamp': u'2012-09-27 14:11:27.086575',
    u'_context_tenant_id': u'82ed0c40ebe64d0bb3310027039c8ed2',
    u'payload': {u'network':
                 {u'status': u'ACTIVE',
                  u'subnets': [],
                  u'name': u'abcedf',
                  u'router:external': False,
                  u'tenant_id': u'82ed0c40ebe64d0bb3310027039c8ed2',
                  u'admin_state_up': True,
                  u'shared': False,
                  u'id': u'7fd4eb2f-a38e-4c25-8490-71ca8800c9be'}},
    u'priority': u'INFO',
    u'_context_is_admin': False,
    u'_context_timestamp': u'2012-09-27 14:11:26.924779',
    u'_context_user_id': u'b44b7ce67fc84414a5c1660a92a1b862',
    u'publisher_id': u'network.ubuntu-VirtualBox',
    u'message_id': u'9e839576-cc47-4c60-a7d8-5743681213b1'}

NOTIFICATION_BULK_NETWORK_CREATE = {
    '_context_roles': [u'_member_',
                       u'heat_stack_owner',
                       u'admin'],
    u'_context_request_id': u'req-a2dfdefd-b773-4400-9d52-5e146e119950',
    u'_context_read_deleted': u'no',
    u'event_type': u'network.create.end',
    u'_context_user_name': u'admin',
    u'_context_project_name': u'admin',
    u'timestamp': u'2014-05-1510: 24: 56.335612',
    u'_context_tenant_id': u'980ec4870033453ead65c0470a78b8a8',
    u'_context_tenant_name': u'admin',
    u'_context_tenant': u'980ec4870033453ead65c0470a78b8a8',
    u'message_id': u'914eb601-9390-4a72-8629-f013a4c84467',
    u'priority': 'info',
    u'_context_is_admin': True,
    u'_context_project_id': u'980ec4870033453ead65c0470a78b8a8',
    u'_context_timestamp': u'2014-05-1510: 24: 56.285975',
    u'_context_user': u'7520940056d54cceb25cbce888300bea',
    u'_context_user_id': u'7520940056d54cceb25cbce888300bea',
    u'publisher_id': u'network.devstack',
    u'payload': {
        u'networks': [{u'status': u'ACTIVE',
                       u'subnets': [],
                       u'name': u'test2',
                       u'provider: physical_network': None,
                       u'admin_state_up': True,
                       u'tenant_id': u'980ec4870033453ead65c0470a78b8a8',
                       u'provider: network_type': u'local',
                       u'shared': False,
                       u'id': u'7cbc7a66-bbd0-41fc-a186-81c3da5c9843',
                       u'provider: segmentation_id': None},
                      {u'status': u'ACTIVE',
                       u'subnets': [],
                       u'name': u'test3',
                       u'provider: physical_network': None,
                       u'admin_state_up': True,
                       u'tenant_id': u'980ec4870033453ead65c0470a78b8a8',
                       u'provider: network_type': u'local',
                       u'shared': False,
                       u'id': u'5a7cb86f-1638-4cc1-8dcc-8bbbc8c7510d',
                       u'provider: segmentation_id': None}]
    }
}

NOTIFICATION_SUBNET_CREATE = {
    u'_context_roles': [u'anotherrole',
                        u'Member'],
    u'_context_read_deleted': u'no',
    u'event_type': u'subnet.create.end',
    u'timestamp': u'2012-09-27 14:11:27.426620',
    u'_context_tenant_id': u'82ed0c40ebe64d0bb3310027039c8ed2',
    u'payload': {
        u'subnet': {
            u'name': u'mysubnet',
            u'enable_dhcp': True,
            u'network_id': u'7fd4eb2f-a38e-4c25-8490-71ca8800c9be',
            u'tenant_id': u'82ed0c40ebe64d0bb3310027039c8ed2',
            u'dns_nameservers': [],
            u'allocation_pools': [{u'start': u'192.168.42.2',
                                   u'end': u'192.168.42.254'}],
            u'host_routes': [],
            u'ip_version': 4,
            u'gateway_ip': u'192.168.42.1',
            u'cidr': u'192.168.42.0/24',
            u'id': u'1a3a170d-d7ce-4cc9-b1db-621da15a25f5'}},
    u'priority': u'INFO',
    u'_context_is_admin': False,
    u'_context_timestamp': u'2012-09-27 14:11:27.214490',
    u'_context_user_id': u'b44b7ce67fc84414a5c1660a92a1b862',
    u'publisher_id': u'network.ubuntu-VirtualBox',
    u'message_id': u'd86dfc66-d3c3-4aea-b06d-bf37253e6116'}

NOTIFICATION_BULK_SUBNET_CREATE = {
    '_context_roles': [u'_member_',
                       u'heat_stack_owner',
                       u'admin'],
    u'_context_request_id': u'req-b77e278a-0cce-4987-9f82-15957b234768',
    u'_context_read_deleted': u'no',
    u'event_type': u'subnet.create.end',
    u'_context_user_name': u'admin',
    u'_context_project_name': u'admin',
    u'timestamp': u'2014-05-1510: 47: 08.133888',
    u'_context_tenant_id': u'980ec4870033453ead65c0470a78b8a8',
    u'_context_tenant_name': u'admin',
    u'_context_tenant': u'980ec4870033453ead65c0470a78b8a8',
    u'message_id': u'c7e6f9fd-ead2-415f-8493-b95bedf72e43',
    u'priority': u'info',
    u'_context_is_admin': True,
    u'_context_project_id': u'980ec4870033453ead65c0470a78b8a8',
    u'_context_timestamp': u'2014-05-1510: 47: 07.970043',
    u'_context_user': u'7520940056d54cceb25cbce888300bea',
    u'_context_user_id': u'7520940056d54cceb25cbce888300bea',
    u'publisher_id': u'network.devstack',
    u'payload': {
        u'subnets': [{u'name': u'',
                      u'enable_dhcp': True,
                      u'network_id': u'3ddfe60b-34b4-4e9d-9440-43c904b1c58e',
                      u'tenant_id': u'980ec4870033453ead65c0470a78b8a8',
                      u'dns_nameservers': [],
                      u'ipv6_ra_mode': None,
                      u'allocation_pools': [{u'start': u'10.0.4.2',
                                             u'end': u'10.0.4.254'}],
                      u'host_routes': [],
                      u'ipv6_address_mode': None,
                      u'ip_version': 4,
                      u'gateway_ip': u'10.0.4.1',
                      u'cidr': u'10.0.4.0/24',
                      u'id': u'14020d7b-6dd7-4349-bb8e-8f954c919022'},
                     {u'name': u'',
                      u'enable_dhcp': True,
                      u'network_id': u'3ddfe60b-34b4-4e9d-9440-43c904b1c58e',
                      u'tenant_id': u'980ec4870033453ead65c0470a78b8a8',
                      u'dns_nameservers': [],
                      u'ipv6_ra_mode': None,
                      u'allocation_pools': [{u'start': u'10.0.5.2',
                                             u'end': u'10.0.5.254'}],
                      u'host_routes': [],
                      u'ipv6_address_mode': None,
                      u'ip_version': 4,
                      u'gateway_ip': u'10.0.5.1',
                      u'cidr': u'10.0.5.0/24',
                      u'id': u'a080991b-a32a-4bf7-a558-96c4b77d075c'}]
    }
}

NOTIFICATION_PORT_CREATE = {
    u'_context_roles': [u'anotherrole',
                        u'Member'],
    u'_context_read_deleted': u'no',
    u'event_type': u'port.create.end',
    u'timestamp': u'2012-09-27 14:28:31.536370',
    u'_context_tenant_id': u'82ed0c40ebe64d0bb3310027039c8ed2',
    u'payload': {
        u'port': {
            u'status': u'ACTIVE',
            u'name': u'',
            u'admin_state_up': True,
            u'network_id': u'7fd4eb2f-a38e-4c25-8490-71ca8800c9be',
            u'tenant_id': u'82ed0c40ebe64d0bb3310027039c8ed2',
            u'device_owner': u'',
            u'mac_address': u'fa:16:3e:75:0c:49',
            u'fixed_ips': [{
                u'subnet_id': u'1a3a170d-d7ce-4cc9-b1db-621da15a25f5',
                u'ip_address': u'192.168.42.3'}],
            u'id': u'9cdfeb92-9391-4da7-95a1-ca214831cfdb',
            u'device_id': u''}},
    u'priority': u'INFO',
    u'_context_is_admin': False,
    u'_context_timestamp': u'2012-09-27 14:28:31.438919',
    u'_context_user_id': u'b44b7ce67fc84414a5c1660a92a1b862',
    u'publisher_id': u'network.ubuntu-VirtualBox',
    u'message_id': u'7135b8ab-e13c-4ac8-bc31-75e7f756622a'}

NOTIFICATION_BULK_PORT_CREATE = {
    u'_context_roles': [u'_member_',
                        u'SwiftOperator'],
    u'_context_request_id': u'req-678be9ad-c399-475a-b3e8-8da0c06375aa',
    u'_context_read_deleted': u'no',
    u'event_type': u'port.create.end',
    u'_context_project_name': u'demo',
    u'timestamp': u'2014-05-0909: 19: 58.317548',
    u'_context_tenant_id': u'133087d90fc149528b501dd8b75ea965',
    u'_context_timestamp': u'2014-05-0909: 19: 58.160011',
    u'_context_tenant': u'133087d90fc149528b501dd8b75ea965',
    u'payload': {
        u'ports': [{u'status': u'DOWN',
                    u'name': u'port--1501135095',
                    u'allowed_address_pairs': [],
                    u'admin_state_up': True,
                    u'network_id': u'acf63fdc-b43b-475d-8cca-9429b843d5e8',
                    u'tenant_id': u'133087d90fc149528b501dd8b75ea965',
                    u'binding: vnic_type': u'normal',
                    u'device_owner': u'',
                    u'mac_address': u'fa: 16: 3e: 37: 10: 39',
                    u'fixed_ips': [],
                    u'id': u'296c2c9f-14e9-48da-979d-78b213454c59',
                    u'security_groups': [
                        u'a06f7c9d-9e5a-46b0-9f6c-ce812aa2e5ff'],
                    u'device_id': u''},
                   {u'status': u'DOWN',
                    u'name': u'',
                    u'allowed_address_pairs': [],
                    u'admin_state_up': False,
                    u'network_id': u'0a8eea59-0146-425c-b470-e9ddfa99ec61',
                    u'tenant_id': u'133087d90fc149528b501dd8b75ea965',
                    u'binding: vnic_type': u'normal',
                    u'device_owner': u'',
                    u'mac_address': u'fa: 16: 3e: 8e: 6e: 53',
                    u'fixed_ips': [],
                    u'id': u'd8bb667f-5cd3-4eca-a984-268e25b1b7a5',
                    u'security_groups': [
                        u'a06f7c9d-9e5a-46b0-9f6c-ce812aa2e5ff'],
                    u'device_id': u''}]
    },
    u'_unique_id': u'60b1650f17fc4fa59492f447321fb26c',
    u'_context_is_admin': False,
    u'_context_project_id': u'133087d90fc149528b501dd8b75ea965',
    u'_context_tenant_name': u'demo',
    u'_context_user': u'b1eb48f9c54741f4adc1b4ea512d400c',
    u'_context_user_name': u'demo',
    u'publisher_id': u'network.os-ci-test12',
    u'message_id': u'04aa45e1-3c30-4c69-8638-e7ff8621e9bc',
    u'_context_user_id': u'b1eb48f9c54741f4adc1b4ea512d400c',
    u'priority': u'INFO'
}

NOTIFICATION_PORT_UPDATE = {
    u'_context_roles': [u'anotherrole',
                        u'Member'],
    u'_context_read_deleted': u'no',
    u'event_type': u'port.update.end',
    u'timestamp': u'2012-09-27 14:35:09.514052',
    u'_context_tenant_id': u'82ed0c40ebe64d0bb3310027039c8ed2',
    u'payload': {
        u'port': {
            u'status': u'ACTIVE',
            u'name': u'bonjour',
            u'admin_state_up': True,
            u'network_id': u'7fd4eb2f-a38e-4c25-8490-71ca8800c9be',
            u'tenant_id': u'82ed0c40ebe64d0bb3310027039c8ed2',
            u'device_owner': u'',
            u'mac_address': u'fa:16:3e:75:0c:49',
            u'fixed_ips': [{
                u'subnet_id': u'1a3a170d-d7ce-4cc9-b1db-621da15a25f5',
                u'ip_address': u'192.168.42.3'}],
            u'id': u'9cdfeb92-9391-4da7-95a1-ca214831cfdb',
            u'device_id': u''}},
    u'priority': u'INFO',
    u'_context_is_admin': False,
    u'_context_timestamp': u'2012-09-27 14:35:09.447682',
    u'_context_user_id': u'b44b7ce67fc84414a5c1660a92a1b862',
    u'publisher_id': u'network.ubuntu-VirtualBox',
    u'message_id': u'07b0a3a1-c0b5-40ab-a09c-28dee6bf48f4'}


NOTIFICATION_NETWORK_EXISTS = {
    u'_context_roles': [u'anotherrole',
                        u'Member'],
    u'_context_read_deleted': u'no',
    u'event_type': u'network.exists',
    u'timestamp': u'2012-09-27 14:11:27.086575',
    u'_context_tenant_id': u'82ed0c40ebe64d0bb3310027039c8ed2',
    u'payload': {u'network':
                 {u'status': u'ACTIVE',
                  u'subnets': [],
                  u'name': u'abcedf',
                  u'router:external': False,
                  u'tenant_id': u'82ed0c40ebe64d0bb3310027039c8ed2',
                  u'admin_state_up': True,
                  u'shared': False,
                  u'id': u'7fd4eb2f-a38e-4c25-8490-71ca8800c9be'}},
    u'priority': u'INFO',
    u'_context_is_admin': False,
    u'_context_timestamp': u'2012-09-27 14:11:26.924779',
    u'_context_user_id': u'b44b7ce67fc84414a5c1660a92a1b862',
    u'publisher_id': u'network.ubuntu-VirtualBox',
    u'message_id': u'9e839576-cc47-4c60-a7d8-5743681213b1'}


NOTIFICATION_ROUTER_EXISTS = {
    u'_context_roles': [u'anotherrole',
                        u'Member'],
    u'_context_read_deleted': u'no',
    u'event_type': u'router.exists',
    u'timestamp': u'2012-09-27 14:11:27.086575',
    u'_context_tenant_id': u'82ed0c40ebe64d0bb3310027039c8ed2',
    u'payload': {u'router':
                 {'status': u'ACTIVE',
                  'external_gateway_info':
                  {'network_id': u'89d55642-4dec-43a4-a617-6cec051393b5'},
                  'name': u'router1',
                  'admin_state_up': True,
                  'tenant_id': u'bb04a2b769c94917b57ba49df7783cfd',
                  'id': u'ab8bb3ed-df23-4ca0-8f03-b887abcd5c23'}},
    u'priority': u'INFO',
    u'_context_is_admin': False,
    u'_context_timestamp': u'2012-09-27 14:11:26.924779',
    u'_context_user_id': u'b44b7ce67fc84414a5c1660a92a1b862',
    u'publisher_id': u'network.ubuntu-VirtualBox',
    u'message_id': u'9e839576-cc47-4c60-a7d8-5743681213b1'}


NOTIFICATION_FLOATINGIP_EXISTS = {
    u'_context_roles': [u'anotherrole',
                        u'Member'],
    u'_context_read_deleted': u'no',
    u'event_type': u'floatingip.exists',
    u'timestamp': u'2012-09-27 14:11:27.086575',
    u'_context_tenant_id': u'82ed0c40ebe64d0bb3310027039c8ed2',
    u'payload': {u'floatingip':
                 {'router_id': None,
                  'tenant_id': u'6e5f9df9b3a249ab834f25fe1b1b81fd',
                  'floating_network_id':
                  u'001400f7-1710-4245-98c3-39ba131cc39a',
                  'fixed_ip_address': None,
                  'floating_ip_address': u'172.24.4.227',
                  'port_id': None,
                  'id': u'2b7cc28c-6f78-4735-9246-257168405de6'}},
    u'priority': u'INFO',
    u'_context_is_admin': False,
    u'_context_timestamp': u'2012-09-27 14:11:26.924779',
    u'_context_user_id': u'b44b7ce67fc84414a5c1660a92a1b862',
    u'publisher_id': u'network.ubuntu-VirtualBox',
    u'message_id': u'9e839576-cc47-4c60-a7d8-5743681213b1'}


NOTIFICATION_FLOATINGIP_UPDATE_START = {
    '_context_roles': [u'_member_',
                       u'admin',
                       u'heat_stack_owner'],
    '_context_request_id': u'req-bd5ed336-242f-4705-836e-8e8f3d0d1ced',
    '_context_read_deleted': u'no',
    'event_type': u'floatingip.update.start',
    '_context_user_name': u'admin',
    '_context_project_name': u'admin',
    'timestamp': u'2014-05-3107: 19: 43.463101',
    '_context_tenant_id': u'9fc714821a3747c8bc4e3a9bfbe82732',
    '_context_tenant_name': u'admin',
    '_context_tenant': u'9fc714821a3747c8bc4e3a9bfbe82732',
    'message_id': u'0ab6d71f-ba0a-4501-86fe-6cc20521ef5a',
    'priority': 'info',
    '_context_is_admin': True,
    '_context_project_id': u'9fc714821a3747c8bc4e3a9bfbe82732',
    '_context_timestamp': u'2014-05-3107: 19: 43.460767',
    '_context_user': u'6ca7b13b33e4425cae0b85e2cf93d9a1',
    '_context_user_id': u'6ca7b13b33e4425cae0b85e2cf93d9a1',
    'publisher_id': u'network.devstack',
    'payload': {
        u'id': u'64262b2a-8f5d-4ade-9405-0cbdd03c1555',
        u'floatingip': {
            u'fixed_ip_address': u'172.24.4.227',
            u'port_id': u'8ab815c8-03cc-4b45-a673-79bdd0c258f2'
        }
    }
}


NOTIFICATION_POOL_CREATE = {
    "_context_roles": ["heat_stack_owner", "admin"],
    "_context_request_id": "req-10715057-7590-4529-8020-b994295ee6f4",
    "event_type": "pool.create.end",
    "timestamp": "2014-09-15 17:20:50.687649",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "ce255443233748ce9cc71b480974df28",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "pool": {
            "status": "ACTIVE",
            "lb_method": "ROUND_ROBIN",
            "protocol": "HTTP", "description": "",
            "health_monitors": [],
            "members": [],
            "status_description": None,
            "id": "6d726518-f3aa-4dd4-ac34-e156a35c0aff",
            "vip_id": None,
            "name": "my_pool",
            "admin_state_up": True,
            "subnet_id": "afaf251b-2ec3-42ac-9fa9-82a4195724fa",
            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
            "health_monitors_status": [],
            "provider": "haproxy"}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:20:49.600299",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "0a5ed7a6-e516-4aed-9968-4ee9f1b65cc2"}


NOTIFICATION_VIP_CREATE = {
    "_context_roles": ["heat_stack_owner", "admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "vip.create.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "vip": {
            "status": "ACTIVE",
            "protocol": "HTTP",
            "description": "",
            "address": "10.0.0.2",
            "protocol_port": 80,
            "port_id": "2b5dd476-11da-4d46-9f1e-7a75436062f6",
            "id": "87a5ce35-f278-47f3-8990-7f695f52f9bf",
            "status_description": None,
            "name": "my_vip",
            "admin_state_up": True,
            "subnet_id": "afaf251b-2ec3-42ac-9fa9-82a4195724fa",
            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
            "connection_limit": -1,
            "pool_id": "6d726518-f3aa-4dd4-ac34-e156a35c0aff",
            "session_persistence": {"type": "SOURCE_IP"}}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "3895ad11-98a3-4031-92af-f76e96736661"}


NOTIFICATION_HEALTH_MONITORS_CREATE = {
    "_context_roles": ["heat_stack_owner", "admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "health_monitor.create.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "health_monitor": {
            "admin_state_up": True,
            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
            "delay": 10,
            "max_retries": 10,
            "timeout": 10,
            "pools": [],
            "type": "PING",
            "id": "6dea2d01-c3af-4696-9192-6c938f391f01"}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "65067e3f-830d-4fbb-87e2-f0e51fda83d2"}


NOTIFICATION_MEMBERS_CREATE = {
    "_context_roles": ["heat_stack_owner", "admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "member.create.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "member": {"admin_state_up": True,
                   "status": "ACTIVE",
                   "status_description": None,
                   "weight": 1,
                   "address": "10.0.0.3",
                   "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
                   "protocol_port": 80,
                   "id": "5e32f960-63ae-4a93-bfa2-339aa83d82ce",
                   "pool_id": "6b73b9f8-d807-4553-87df-eb34cdd08070"}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "65067e3f-830d-4fbb-87e2-f0e51fda83d2"}


NOTIFICATION_FIREWALL_CREATE = {
    "_context_roles": ["heat_stack_owner", "admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "firewall.create.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "firewall": {
            "status": "ACTIVE",
            "name": "my_firewall",
            "admin_state_up": True,
            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
            "firewall_policy_id": "c46a1c15-0496-41c9-beff-9a309a25653e",
            "id": "e2d1155f-6bc4-4292-9cfa-ea91af4b38c8",
            "description": ""}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "fdffeca1-2b5a-4dc9-b8ae-87c482a83e0d"}


NOTIFICATION_FIREWALL_RULE_CREATE = {
    "_context_roles": ["heat_stack_owner", "admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "firewall_rule.create.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "firewall_rule": {
            "protocol": "tcp",
            "description": "",
            "source_port": 80,
            "source_ip_address": '192.168.255.10',
            "destination_ip_address": '10.10.10.1',
            "firewall_policy_id": '',
            "position": None,
            "destination_port": 80,
            "id": "53b7c0d3-cb87-4069-9e29-1e866583cc8c",
            "name": "rule_01",
            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
            "enabled": True,
            "action": "allow",
            "ip_version": 4,
            "shared": False}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "fdffeca1-2b5a-4dc9-b8ae-87c482a83e0d"}


NOTIFICATION_FIREWALL_POLICY_CREATE = {
    "_context_roles": ["heat_stack_owner", "admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "firewall_policy.create.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "firewall_policy": {"name": "my_policy",
                            "firewall_rules": [],
                            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
                            "audited": False,
                            "shared": False,
                            "id": "c46a1c15-0496-41c9-beff-9a309a25653e",
                            "description": ""}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "fdffeca1-2b5a-4dc9-b8ae-87c482a83e0d"}


NOTIFICATION_VPNSERVICE_CREATE = {
    "_context_roles": ["heat_stack_owner", "admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "vpnservice.create.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "vpnservice": {"router_id": "75871c53-e722-4b21-93ed-20cb40b6b672",
                       "status": "ACTIVE",
                       "name": "my_vpn",
                       "admin_state_up": True,
                       "subnet_id": "afaf251b-2ec3-42ac-9fa9-82a4195724fa",
                       "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
                       "id": "270c40cc-28d5-4a7e-83da-cc33088ee5d6",
                       "description": ""}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "65067e3f-830d-4fbb-87e2-f0e51fda83d2"}


NOTIFICATION_IPSEC_POLICY_CREATE = {
    "_context_roles": ["heat_stack_owner", "admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "ipsecpolicy.create.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "ipsecpolicy": {"encapsulation_mode": "tunnel",
                        "encryption_algorithm": "aes-128",
                        "pfs": "group5",
                        "lifetime": {
                            "units": "seconds",
                            "value": 3600},
                        "name": "my_ipsec_polixy",
                        "transform_protocol": "esp",
                        "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
                        "id": "998d910d-4506-47c9-a160-47ec51ff53fc",
                        "auth_algorithm": "sha1",
                        "description": ""}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "4c0e6ecb-2e40-4975-aee2-d88045c747bf"}


NOTIFICATION_IKE_POLICY_CREATE = {
    "_context_roles": ["heat_stack_owner", "admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "ikepolicy.create.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "ikepolicy": {"encryption_algorithm": "aes-128",
                      "pfs": "group5",
                      "name": "my_ike_policy",
                      "phase1_negotiation_mode": "main",
                      "lifetime": {"units": "seconds",
                                   "value": 3600},
                      "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
                      "ike_version": "v1",
                      "id": "11cef94e-3f6a-4b65-8058-7deb1838633a",
                      "auth_algorithm": "sha1",
                      "description": ""}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "4c0e6ecb-2e40-4975-aee2-d88045c747bf"}


NOTIFICATION_IPSEC_SITE_CONN_CREATE = {
    "_context_roles": ["heat_stack_owner", "admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "ipsec_site_connection.create.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "ipsec_site_connection": {
            "status": "ACTIVE",
            "psk": "test",
            "initiator": "bi-directional",
            "name": "my_ipsec_connection",
            "admin_state_up": True,
            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
            "ipsecpolicy_id": "998d910d-4506-47c9-a160-47ec51ff53fc",
            "auth_mode": "psk", "peer_cidrs": ["192.168.255.0/24"],
            "mtu": 1500,
            "ikepolicy_id": "11cef94e-3f6a-4b65-8058-7deb1838633a",
            "dpd": {"action": "hold",
                    "interval": 30,
                    "timeout": 120},
            "route_mode": "static",
            "vpnservice_id": "270c40cc-28d5-4a7e-83da-cc33088ee5d6",
            "peer_address": "10.0.0.1",
            "peer_id": "10.0.0.254",
            "id": "06f3c1ec-2e01-4ad6-9c98-4252751fc60a",
            "description": ""}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "4c0e6ecb-2e40-4975-aee2-d88045c747bf"}


NOTIFICATION_POOL_UPDATE = {
    "_context_roles": ["admin"],
    "_context_request_id": "req-10715057-7590-4529-8020-b994295ee6f4",
    "event_type": "pool.update.end",
    "timestamp": "2014-09-15 17:20:50.687649",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "ce255443233748ce9cc71b480974df28",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "pool": {
            "status": "ACTIVE",
            "lb_method": "ROUND_ROBIN",
            "protocol": "HTTP", "description": "",
            "health_monitors": [],
            "members": [],
            "status_description": None,
            "id": "6d726518-f3aa-4dd4-ac34-e156a35c0aff",
            "vip_id": None,
            "name": "my_pool",
            "admin_state_up": True,
            "subnet_id": "afaf251b-2ec3-42ac-9fa9-82a4195724fa",
            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
            "health_monitors_status": [],
            "provider": "haproxy"}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:20:49.600299",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "0a5ed7a6-e516-4aed-9968-4ee9f1b65cc2"}


NOTIFICATION_VIP_UPDATE = {
    "_context_roles": ["admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "vip.update.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "vip": {
            "status": "ACTIVE",
            "protocol": "HTTP",
            "description": "",
            "address": "10.0.0.2",
            "protocol_port": 80,
            "port_id": "2b5dd476-11da-4d46-9f1e-7a75436062f6",
            "id": "87a5ce35-f278-47f3-8990-7f695f52f9bf",
            "status_description": None,
            "name": "my_vip",
            "admin_state_up": True,
            "subnet_id": "afaf251b-2ec3-42ac-9fa9-82a4195724fa",
            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
            "connection_limit": -1,
            "pool_id": "6d726518-f3aa-4dd4-ac34-e156a35c0aff",
            "session_persistence": {"type": "SOURCE_IP"}}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "3895ad11-98a3-4031-92af-f76e96736661"}


NOTIFICATION_HEALTH_MONITORS_UPDATE = {
    "_context_roles": ["admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "health_monitor.update.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "health_monitor": {
            "admin_state_up": True,
            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
            "delay": 10,
            "max_retries": 10,
            "timeout": 10,
            "pools": [],
            "type": "PING",
            "id": "6dea2d01-c3af-4696-9192-6c938f391f01"}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "65067e3f-830d-4fbb-87e2-f0e51fda83d2"}


NOTIFICATION_MEMBERS_UPDATE = {
    "_context_roles": ["admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "member.update.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "member": {"admin_state_up": True,
                   "status": "ACTIVE",
                   "status_description": None,
                   "weight": 1,
                   "address": "10.0.0.3",
                   "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
                   "protocol_port": 80,
                   "id": "5e32f960-63ae-4a93-bfa2-339aa83d82ce",
                   "pool_id": "6b73b9f8-d807-4553-87df-eb34cdd08070"}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "65067e3f-830d-4fbb-87e2-f0e51fda83d2"}


NOTIFICATION_FIREWALL_UPDATE = {
    "_context_roles": ["admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "firewall.update.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "firewall": {
            "status": "ACTIVE",
            "name": "my_firewall",
            "admin_state_up": True,
            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
            "firewall_policy_id": "c46a1c15-0496-41c9-beff-9a309a25653e",
            "id": "e2d1155f-6bc4-4292-9cfa-ea91af4b38c8",
            "description": ""}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "fdffeca1-2b5a-4dc9-b8ae-87c482a83e0d"}


NOTIFICATION_FIREWALL_RULE_UPDATE = {
    "_context_roles": ["admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "firewall_rule.update.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "firewall_rule": {
            "protocol": "tcp",
            "description": "",
            "source_port": 80,
            "source_ip_address": '192.168.255.10',
            "destination_ip_address": '10.10.10.1',
            "firewall_policy_id": '',
            "position": None,
            "destination_port": 80,
            "id": "53b7c0d3-cb87-4069-9e29-1e866583cc8c",
            "name": "rule_01",
            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
            "enabled": True,
            "action": "allow",
            "ip_version": 4,
            "shared": False}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "fdffeca1-2b5a-4dc9-b8ae-87c482a83e0d"}


NOTIFICATION_FIREWALL_POLICY_UPDATE = {
    "_context_roles": ["admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "firewall_policy.update.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "firewall_policy": {"name": "my_policy",
                            "firewall_rules": [],
                            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
                            "audited": False,
                            "shared": False,
                            "id": "c46a1c15-0496-41c9-beff-9a309a25653e",
                            "description": ""}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "fdffeca1-2b5a-4dc9-b8ae-87c482a83e0d"}


NOTIFICATION_VPNSERVICE_UPDATE = {
    "_context_roles": ["admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "vpnservice.update.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "vpnservice": {"router_id": "75871c53-e722-4b21-93ed-20cb40b6b672",
                       "status": "ACTIVE",
                       "name": "my_vpn",
                       "admin_state_up": True,
                       "subnet_id": "afaf251b-2ec3-42ac-9fa9-82a4195724fa",
                       "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
                       "id": "270c40cc-28d5-4a7e-83da-cc33088ee5d6",
                       "description": ""}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "65067e3f-830d-4fbb-87e2-f0e51fda83d2"}


NOTIFICATION_IPSEC_POLICY_UPDATE = {
    "_context_roles": ["admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "ipsecpolicy.update.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "ipsecpolicy": {"encapsulation_mode": "tunnel",
                        "encryption_algorithm": "aes-128",
                        "pfs": "group5",
                        "lifetime": {
                            "units": "seconds",
                            "value": 3600},
                        "name": "my_ipsec_polixy",
                        "transform_protocol": "esp",
                        "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
                        "id": "998d910d-4506-47c9-a160-47ec51ff53fc",
                        "auth_algorithm": "sha1",
                        "description": ""}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "4c0e6ecb-2e40-4975-aee2-d88045c747bf"}


NOTIFICATION_IKE_POLICY_UPDATE = {
    "_context_roles": ["admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "ikepolicy.update.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "ikepolicy": {"encryption_algorithm": "aes-128",
                      "pfs": "group5",
                      "name": "my_ike_policy",
                      "phase1_negotiation_mode": "main",
                      "lifetime": {"units": "seconds",
                                   "value": 3600},
                      "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
                      "ike_version": "v1",
                      "id": "11cef94e-3f6a-4b65-8058-7deb1838633a",
                      "auth_algorithm": "sha1",
                      "description": ""}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "4c0e6ecb-2e40-4975-aee2-d88045c747bf"}


NOTIFICATION_IPSEC_SITE_CONN_UPDATE = {
    "_context_roles": ["admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "ipsec_site_connection.update.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "ipsec_site_connection": {
            "status": "ACTIVE",
            "psk": "test",
            "initiator": "bi-directional",
            "name": "my_ipsec_connection",
            "admin_state_up": True,
            "tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
            "ipsecpolicy_id": "998d910d-4506-47c9-a160-47ec51ff53fc",
            "auth_mode": "psk", "peer_cidrs": ["192.168.255.0/24"],
            "mtu": 1500,
            "ikepolicy_id": "11cef94e-3f6a-4b65-8058-7deb1838633a",
            "dpd": {"action": "hold",
                    "interval": 30,
                    "timeout": 120},
            "route_mode": "static",
            "vpnservice_id": "270c40cc-28d5-4a7e-83da-cc33088ee5d6",
            "peer_address": "10.0.0.1",
            "peer_id": "10.0.0.254",
            "id": "06f3c1ec-2e01-4ad6-9c98-4252751fc60a",
            "description": ""}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "4c0e6ecb-2e40-4975-aee2-d88045c747bf"}

NOTIFICATION_EMPTY_PAYLOAD = {
    "_context_roles": ["heat_stack_owner", "admin"],
    "_context_request_id": "req-e56a8a5e-5d42-43e8-9677-2d36e6e17d5e",
    "event_type": "health_monitor.create.end",
    "timestamp": "2014-09-15 17:22:11.323644",
    "_context_tenant_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_user": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "_unique_id": "f112a185e1d1424eba3a13df9e0f0277",
    "_context_tenant_name": "demo",
    "_context_user_id": "1c1f7c80efc24a16b835ae1c0802d0a1",
    "payload": {
        "health_monitor": {}},
    "_context_project_name": "demo",
    "_context_read_deleted": "no",
    "_context_auth_token": "e6daf56d7d1787e1fbefff0ecf29703f",
    "_context_tenant": "a820f2d6293b4a7587d1c582767f43fb",
    "priority": "INFO",
    "_context_is_admin": True,
    "_context_project_id": "a820f2d6293b4a7587d1c582767f43fb",
    "_context_timestamp": "2014-09-15 17:22:11.187163",
    "_context_user_name": "admin",
    "publisher_id": "network.ubuntu",
    "message_id": "65067e3f-830d-4fbb-87e2-f0e51fda83d2"}


class TestNotifications(test.BaseTestCase):
    def test_network_create(self):
        v = notifications.Network(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_NETWORK_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.create", samples[1].name)

    def test_bulk_network_create(self):
        v = notifications.Network(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_BULK_NETWORK_CREATE))
        self.assertEqual(4, len(samples))
        self.assertEqual("network", samples[0].name)
        self.assertEqual("network.create", samples[1].name)
        self.assertEqual("network", samples[2].name)
        self.assertEqual("network.create", samples[3].name)

    def test_subnet_create(self):
        v = notifications.Subnet(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_SUBNET_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("subnet.create", samples[1].name)

    def test_bulk_subnet_create(self):
        v = notifications.Subnet(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_BULK_SUBNET_CREATE))
        self.assertEqual(4, len(samples))
        self.assertEqual("subnet", samples[0].name)
        self.assertEqual("subnet.create", samples[1].name)
        self.assertEqual("subnet", samples[2].name)
        self.assertEqual("subnet.create", samples[3].name)

    def test_port_create(self):
        v = notifications.Port(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_PORT_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("port.create", samples[1].name)

    def test_bulk_port_create(self):
        v = notifications.Port(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_BULK_PORT_CREATE))
        self.assertEqual(4, len(samples))
        self.assertEqual("port", samples[0].name)
        self.assertEqual("port.create", samples[1].name)
        self.assertEqual("port", samples[2].name)
        self.assertEqual("port.create", samples[3].name)

    def test_port_update(self):
        v = notifications.Port(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_PORT_UPDATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("port.update", samples[1].name)

    def test_network_exists(self):
        v = notifications.Network(mock.Mock())
        samples = v.process_notification(NOTIFICATION_NETWORK_EXISTS)
        self.assertEqual(1, len(list(samples)))

    def test_router_exists(self):
        v = notifications.Router(mock.Mock())
        samples = v.process_notification(NOTIFICATION_ROUTER_EXISTS)
        self.assertEqual(1, len(list(samples)))

    def test_floatingip_exists(self):
        v = notifications.FloatingIP(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_FLOATINGIP_EXISTS))
        self.assertEqual(1, len(samples))
        self.assertEqual("ip.floating", samples[0].name)

    def test_floatingip_update(self):
        v = notifications.FloatingIP(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_FLOATINGIP_UPDATE_START))
        self.assertEqual(len(samples), 2)
        self.assertEqual("ip.floating", samples[0].name)

    def test_pool_create(self):
        v = notifications.Pool(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_POOL_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.lb.pool", samples[0].name)

    def test_vip_create(self):
        v = notifications.Vip(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_VIP_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.lb.vip", samples[0].name)

    def test_member_create(self):
        v = notifications.Member(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_MEMBERS_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.lb.member", samples[0].name)

    def test_health_monitor_create(self):
        v = notifications.HealthMonitor(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_HEALTH_MONITORS_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.lb.health_monitor", samples[0].name)

    def test_firewall_create(self):
        v = notifications.Firewall(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_FIREWALL_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.firewall", samples[0].name)

    def test_vpnservice_create(self):
        v = notifications.VPNService(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_VPNSERVICE_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.vpn", samples[0].name)

    def test_ipsec_connection_create(self):
        v = notifications.IPSecSiteConnection(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_IPSEC_SITE_CONN_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.vpn.connections", samples[0].name)

    def test_firewall_policy_create(self):
        v = notifications.FirewallPolicy(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_FIREWALL_POLICY_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.firewall.policy", samples[0].name)

    def test_firewall_rule_create(self):
        v = notifications.FirewallRule(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_FIREWALL_RULE_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.firewall.rule", samples[0].name)

    def test_ipsec_policy_create(self):
        v = notifications.IPSecPolicy(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_IPSEC_POLICY_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.vpn.ipsecpolicy", samples[0].name)

    def test_ike_policy_create(self):
        v = notifications.IKEPolicy(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_IKE_POLICY_CREATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.vpn.ikepolicy", samples[0].name)

    def test_pool_update(self):
        v = notifications.Pool(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_POOL_UPDATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.lb.pool", samples[0].name)

    def test_vip_update(self):
        v = notifications.Vip(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_VIP_UPDATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.lb.vip", samples[0].name)

    def test_member_update(self):
        v = notifications.Member(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_MEMBERS_UPDATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.lb.member", samples[0].name)

    def test_health_monitor_update(self):
        v = notifications.HealthMonitor(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_HEALTH_MONITORS_UPDATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.lb.health_monitor", samples[0].name)

    def test_firewall_update(self):
        v = notifications.Firewall(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_FIREWALL_UPDATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.firewall", samples[0].name)

    def test_vpnservice_update(self):
        v = notifications.VPNService(mock.Mock())
        samples = list(v.process_notification(NOTIFICATION_VPNSERVICE_UPDATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.vpn", samples[0].name)

    def test_ipsec_connection_update(self):
        v = notifications.IPSecSiteConnection(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_IPSEC_SITE_CONN_UPDATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.vpn.connections", samples[0].name)

    def test_firewall_policy_update(self):
        v = notifications.FirewallPolicy(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_FIREWALL_POLICY_UPDATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.firewall.policy", samples[0].name)

    def test_firewall_rule_update(self):
        v = notifications.FirewallRule(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_FIREWALL_RULE_UPDATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.firewall.rule", samples[0].name)

    def test_ipsec_policy_update(self):
        v = notifications.IPSecPolicy(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_IPSEC_POLICY_UPDATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.vpn.ipsecpolicy", samples[0].name)

    def test_ike_policy_update(self):
        v = notifications.IKEPolicy(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_IKE_POLICY_UPDATE))
        self.assertEqual(2, len(samples))
        self.assertEqual("network.services.vpn.ikepolicy", samples[0].name)

    def test_empty_event_payload(self):
        v = notifications.HealthMonitor(mock.Mock())
        samples = list(v.process_notification(
            NOTIFICATION_EMPTY_PAYLOAD))
        self.assertEqual(0, len(samples))


class TestEventTypes(test.BaseTestCase):

    def test_network(self):
        v = notifications.Network(mock.Mock())
        events = v.event_types
        self.assertIsNotEmpty(events)

    def test_subnet(self):
        v = notifications.Subnet(mock.Mock())
        events = v.event_types
        self.assertIsNotEmpty(events)

    def test_port(self):
        v = notifications.Port(mock.Mock())
        events = v.event_types
        self.assertIsNotEmpty(events)

    def test_router(self):
        self.assertTrue(notifications.Router(mock.Mock()).event_types)

    def test_floatingip(self):
        self.assertTrue(notifications.FloatingIP(mock.Mock()).event_types)

    def test_pool(self):
        self.assertTrue(notifications.Pool(mock.Mock()).event_types)

    def test_vip(self):
        self.assertTrue(notifications.Vip(mock.Mock()).event_types)

    def test_member(self):
        self.assertTrue(notifications.Member(mock.Mock()).event_types)

    def test_health_monitor(self):
        self.assertTrue(notifications.HealthMonitor(mock.Mock()).event_types)

    def test_firewall(self):
        self.assertTrue(notifications.Firewall(mock.Mock()).event_types)

    def test_vpnservice(self):
        self.assertTrue(notifications.VPNService(mock.Mock()).event_types)

    def test_ipsec_connection(self):
        self.assertTrue(notifications.IPSecSiteConnection(
            mock.Mock()).event_types)

    def test_firewall_policy(self):
        self.assertTrue(notifications.FirewallPolicy(mock.Mock()).event_types)

    def test_firewall_rule(self):
        self.assertTrue(notifications.FirewallRule(mock.Mock()).event_types)

    def test_ipsec_policy(self):
        self.assertTrue(notifications.IPSecPolicy(mock.Mock()).event_types)

    def test_ike_policy(self):
        self.assertTrue(notifications.IKEPolicy(mock.Mock()).event_types)



#
# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json

import six
from sqlalchemy import Boolean
from sqlalchemy import Column
from sqlalchemy import Float
from sqlalchemy import ForeignKey
from sqlalchemy import Integer
from sqlalchemy import MetaData
from sqlalchemy.sql import select
from sqlalchemy import String
from sqlalchemy import Table
from sqlalchemy import Text

from ceilometer import utils

tables = [('metadata_text', Text, True),
          ('metadata_bool', Boolean, False),
          ('metadata_int', Integer, False),
          ('metadata_float', Float, False)]


def upgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    meter = Table('meter', meta, autoload=True)
    meta_tables = {}
    for t_name, t_type, t_nullable in tables:
        meta_tables[t_name] = Table(
            t_name, meta,
            Column('id', Integer, ForeignKey('meter.id'), primary_key=True),
            Column('meta_key', String(255), index=True, primary_key=True),
            Column('value', t_type, nullable=t_nullable),
            mysql_engine='InnoDB',
            mysql_charset='utf8',
        )
        meta_tables[t_name].create()

    for row in select([meter]).execute():
        if row['resource_metadata']:
            meter_id = row['id']
            rmeta = json.loads(row['resource_metadata'])
            for key, v in utils.dict_to_keyval(rmeta):
                ins = None
                if isinstance(v, six.string_types) or v is None:
                    ins = meta_tables['metadata_text'].insert()
                elif isinstance(v, bool):
                    ins = meta_tables['metadata_bool'].insert()
                elif isinstance(v, six.integer_types):
                    ins = meta_tables['metadata_int'].insert()
                elif isinstance(v, float):
                    ins = meta_tables['metadata_float'].insert()
                if ins is not None:
                    ins.values(id=meter_id, meta_key=key, value=v).execute()



# Copyright (c) 2016 OpenStack Foundation
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Guidelines for writing new hacking checks

 - Use only for Ceilometer specific tests. OpenStack general tests
   should be submitted to the common 'hacking' module.
 - Pick numbers in the range X3xx. Find the current test with
   the highest allocated number and then pick the next value.
 - Keep the test method code in the source file ordered based
   on the C3xx value.
 - List the new rule in the top level HACKING.rst file

"""


def no_log_warn(logical_line):
    """Disallow 'LOG.warn('

    https://bugs.launchpad.net/tempest/+bug/1508442

    C301
    """
    if logical_line.startswith('LOG.warn('):
        yield(0, 'C301 Use LOG.warning() rather than LOG.warn()')


def no_os_popen(logical_line):
    """Disallow 'os.popen('

    Deprecated library function os.popen() Replace it using subprocess
    https://bugs.launchpad.net/tempest/+bug/1529836

    C302
    """

    if 'os.popen(' in logical_line:
        yield(0, 'C302 Deprecated library function os.popen(). '
                 'Replace it using subprocess module. ')


def factory(register):
    register(no_log_warn)
    register(no_os_popen)
